@@ -1,3 +1,20 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import re
+import time
+
+from stgit import argparse, stack, git
+from stgit.argparse import opt
+from stgit.commands.common import (CmdException,
+                                   DirectoryGotoToplevel,
+                                   check_local_changes,
+                                   check_conflicts,
+                                   check_head_top_equal,
+                                   git_id)
+from stgit.lib import log
+from stgit.out import out
+
 __copyright__ = """
 Copyright (C) 2005, Chuck Lever <cel@netapp.com>
 
@@ -14,14 +31,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os, time, re
-from stgit.argparse import opt
-from stgit.commands.common import *
-from stgit.utils import *
-from stgit.out import *
-from stgit import argparse, stack, git, basedir
-from stgit.lib import log
-
 help = 'Branch operations: switch, list, create, rename, delete, ...'
 kind = 'stack'
 usage = ['',
@@ -117,7 +126,9 @@ options = [
     opt('--force', action = 'store_true',
         short = 'Force a delete when the series is not empty')]
 
-directory = DirectoryGotoToplevel(log = False)
+directory = DirectoryGotoToplevel(log=False)
+crt_series = None
+
 
 def __is_current_branch(branch_name):
     return crt_series.get_name() == branch_name
@@ -144,7 +155,7 @@ def __delete_branch(doomed_name, force = False):
     if __is_current_branch(doomed_name):
         raise CmdException('Cannot delete the current branch')
     if doomed.get_protected():
-        raise CmdException, 'This branch is protected. Delete is not permitted'
+        raise CmdException('This branch is protected. Delete is not permitted')
 
     out.start('Deleting branch "%s"' % doomed_name)
     doomed.delete(force)
@@ -184,7 +195,7 @@ def func(parser, options, args):
                         # args[1] is a valid ref from the branchpoint
                         # setting above
                         parentbranch = args[1]
-                        break;
+                        break
             except git.GitException:
                 # should use a more specific exception to catch only
                 # non-git refs ?
@@ -296,8 +307,8 @@ def func(parser, options, args):
         branch = stack.Series(branch_name)
 
         if not branch.is_initialised():
-            raise CmdException, 'Branch "%s" is not controlled by StGIT' \
-                  % branch_name
+            raise CmdException('Branch "%s" is not controlled by StGIT' %
+                               branch_name)
 
         out.start('Protecting branch "%s"' % branch_name)
         branch.protect()
@@ -311,7 +322,7 @@ def func(parser, options, args):
             parser.error('incorrect number of arguments')
 
         if __is_current_branch(args[0]):
-            raise CmdException, 'Renaming the current branch is not supported'
+            raise CmdException('Renaming the current branch is not supported')
 
         stack.Series(args[0]).rename(args[1])
 
@@ -330,8 +341,8 @@ def func(parser, options, args):
         branch = stack.Series(branch_name)
 
         if not branch.is_initialised():
-            raise CmdException, 'Branch "%s" is not controlled by StGIT' \
-                  % branch_name
+            raise CmdException('Branch "%s" is not controlled by StGIT' %
+                               branch_name)
 
         out.info('Unprotecting branch "%s"' % branch_name)
         branch.unprotect()
@@ -350,8 +361,8 @@ def func(parser, options, args):
         branch = stack.Series(branch_name)
 
         if not branch.is_initialised():
-            raise CmdException, 'Branch "%s" is not controlled by StGIT' \
-                  % branch_name
+            raise CmdException('Branch "%s" is not controlled by StGIT' %
+                               branch_name)
 
         branch.set_description(options.description)
 
@@ -360,8 +371,8 @@ def func(parser, options, args):
     elif len(args) == 1:
 
         if __is_current_branch(args[0]):
-            raise CmdException, 'Branch "%s" is already the current branch' \
-                  % args[0]
+            raise CmdException('Branch "%s" is already the current branch' %
+                               args[0])
 
         if not options.merge:
             check_local_changes()
@@ -377,4 +388,4 @@ def func(parser, options, args):
     if len(args) != 0:
         parser.error('incorrect number of arguments')
 
-    print crt_series.get_name()
+    print(crt_series.get_name())
