@@ -1,3 +1,11 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit.argparse import opt
+from stgit.commands import common
+from stgit.lib import transaction
+
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,11 +22,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-from stgit.argparse import opt
-from stgit.out import *
-from stgit.commands import common
-from stgit.lib import transaction
-
 help = 'Delete the empty patches in the series'
 kind = 'stack'
 usage = ['']
