@@ -1,3 +1,12 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import os
+
+from stgit import argparse
+from stgit.commands import common
+from stgit.lib import git, stack
+
 __copyright__ = """
 Copyright (C) 2009, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,12 +23,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import os
-from stgit.commands import common
-from stgit.lib import git, stack
-from stgit import argparse
-from stgit.out import out
-
 help = 'Make a local clone of a remote repository'
 kind = 'repo'
 usage = ['<repository> <dir>']
