@@ -1,5 +1,28 @@
-"""Function/variables common to all the commands
-"""
+# -*- coding: utf-8 -*-
+"""Function/variables common to all the commands"""
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import email.utils
+import os
+import re
+import sys
+
+from stgit import stack, git, templates
+from stgit.compat import text, decode_utf8_with_latin1
+from stgit.config import config
+from stgit.exception import StgException
+from stgit.lib import git as libgit
+from stgit.lib import log
+from stgit.lib import stack as libstack
+from stgit.out import out
+from stgit.run import Run, RunException
+from stgit.utils import (EditorException,
+                         add_sign_line,
+                         edit_string,
+                         get_hook,
+                         parse_name_email_date,
+                         run_hook_on_string,
+                         strip_prefix)
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -17,16 +40,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os, os.path, re, email.Utils
-from stgit.exception import *
-from stgit.utils import *
-from stgit.out import *
-from stgit.run import *
-from stgit import stack, git, basedir
-from stgit.config import config, file_extensions
-from stgit.lib import stack as libstack
-from stgit.lib import git as libgit
-from stgit.lib import log
 
 # Command exception class
 class CmdException(StgException):
@@ -133,9 +146,6 @@ def print_crt_patch(crt_series, branch = None):
     else:
         out.info('No patches applied')
 
-def resolved_all(reset = None):
-    conflicts = git.get_conflicts()
-    git.resolved(conflicts, reset)
 
 def push_patches(crt_series, patches, check_merged = False):
     """Push multiple patches onto the stack. This function is shared
@@ -192,10 +202,10 @@ def pop_patches(crt_series, patches, keep = False):
         out.done()
 
 def get_patch_from_list(part_name, patch_list):
-    candidates = [full for full in patch_list if str.find(full, part_name) != -1]
+    candidates = [full for full in patch_list if part_name in full]
     if len(candidates) >= 2:
         out.info('Possible patches:\n  %s' % '\n  '.join(candidates))
-        raise CmdException, 'Ambiguous patch name "%s"' % part_name
+        raise CmdException('Ambiguous patch name "%s"' % part_name)
     elif len(candidates) == 1:
         return candidates[0]
     else:
@@ -213,8 +223,8 @@ def parse_patches(patch_args, patch_list, boundary = 0, ordered = False):
     for name in patch_args:
         pair = name.split('..')
         for p in pair:
-            if p and not p in patch_list:
-                raise CmdException, 'Unknown patch name: %s' % p
+            if p and p not in patch_list:
+                raise CmdException('Unknown patch name: %s' % p)
 
         if len(pair) == 1:
             # single patch name
@@ -252,11 +262,11 @@ def parse_patches(patch_args, patch_list, boundary = 0, ordered = False):
                 pl = patch_list[(last - 1):(first + 1)]
                 pl.reverse()
         else:
-            raise CmdException, 'Malformed patch name: %s' % name
+            raise CmdException('Malformed patch name: %s' % name)
 
         for p in pl:
             if p in patches:
-                raise CmdException, 'Duplicate patch name: %s' % p
+                raise CmdException('Duplicate patch name: %s' % p)
 
         patches += pl
 
@@ -266,7 +276,7 @@ def parse_patches(patch_args, patch_list, boundary = 0, ordered = False):
     return patches
 
 def name_email(address):
-    p = email.Utils.parseaddr(address)
+    p = email.utils.parseaddr(address)
     if p[1]:
         return p
     else:
@@ -292,7 +302,7 @@ def address_or_alias(addr_pair):
     if alias:
         # it's an alias
         return name_email(alias)
-    raise CmdException, 'unknown e-mail alias: %s' % addr
+    raise CmdException('unknown e-mail alias: %s' % addr)
 
 def prepare_rebase(crt_series):
     # pop all patches
@@ -328,23 +338,25 @@ def post_rebase(crt_series, applied, nopush, merged):
 # Patch description/e-mail/diff parsing
 #
 def __end_descr(line):
-    return re.match('---\s*$', line) or re.match('diff -', line) or \
-            re.match('Index: ', line) or re.match('--- \w', line)
+    return (re.match(br'---\s*$', line) or
+            re.match(b'diff -', line) or
+            re.match(b'Index: ', line) or
+            re.match(br'--- \w', line))
 
 def __split_descr_diff(string):
     """Return the description and the diff from the given string
     """
-    descr = diff = ''
+    descr = diff = b''
     top = True
 
-    for line in string.split('\n'):
+    for line in string.split(b'\n'):
         if top:
             if not __end_descr(line):
-                descr += line + '\n'
+                descr += line + b'\n'
                 continue
             else:
                 top = False
-        diff += line + '\n'
+        diff += line + b'\n'
 
     return (descr.rstrip(), diff)
 
@@ -357,7 +369,7 @@ def __parse_description(descr):
 
     descr_lines = [line.rstrip() for line in  descr.split('\n')]
     if not descr_lines:
-        raise CmdException, "Empty patch description"
+        raise CmdException("Empty patch description")
 
     lasthdr = 0
     end = len(descr_lines)
@@ -368,21 +380,21 @@ def __parse_description(descr):
         if not descr_lines[pos]:
            continue
         # check for a "From|Author:" line
-        if re.match('\s*(?:from|author):\s+', descr_lines[pos], re.I):
-            auth = re.findall('^.*?:\s+(.*)$', descr_lines[pos])[0]
+        if re.match(r'\s*(?:from|author):\s+', descr_lines[pos], re.I):
+            auth = re.findall(r'^.*?:\s+(.*)$', descr_lines[pos])[0]
             authname, authemail = name_email(auth)
             lasthdr = pos + 1
             continue
         # check for a "Date:" line
-        if re.match('\s*date:\s+', descr_lines[pos], re.I):
-            authdate = re.findall('^.*?:\s+(.*)$', descr_lines[pos])[0]
+        if re.match(r'\s*date:\s+', descr_lines[pos], re.I):
+            authdate = re.findall(r'^.*?:\s+(.*)$', descr_lines[pos])[0]
             lasthdr = pos + 1
             continue
         if subject:
             break
         # get the subject
         subject = descr_lines[pos][descr_strip:]
-        if re.match('commit [\da-f]{40}$', subject):
+        if re.match(r'commit [\da-f]{40}$', subject):
             # 'git show' output, look for the real subject
             subject = ''
             descr_strip = 4
@@ -398,19 +410,32 @@ def parse_mail(msg):
     """Parse the message object and return (description, authname,
     authemail, authdate, diff)
     """
-    from email.Header import decode_header, make_header
+    import email.header
+    if sys.version_info[0] <= 2:
+        # Python 2's decode_header() fails to decode encoded words if they are
+        # quoted. This does not match the behavior of Python3 or `git mailinfo`.
+        # For example, Python2 does not handle this header correctly:
+        #
+        #    From: "=?UTF-8?q?Christian=20K=C3=B6nig?=" <name@example.com>
+        #
+        # By replacing the encoded words regex in the email.header module, we can
+        # bless Python2 with the same behavior as Python3.
+        email.header.ecre = re.compile(
+            (r'=\? (?P<charset>[^?]*?)'
+             r' \? (?P<encoding>[QqBb])'
+             r' \? (?P<encoded>.*?)'
+             r' \?='), re.VERBOSE | re.MULTILINE)
 
     def __decode_header(header):
         """Decode a qp-encoded e-mail header as per rfc2047"""
         try:
-            words_enc = decode_header(header)
-            hobj = make_header(words_enc)
-        except Exception, ex:
-            raise CmdException, 'header decoding error: %s' % str(ex)
-        return unicode(hobj).encode('utf-8')
+            decoded_words = email.header.decode_header(header)
+            return text(email.header.make_header(decoded_words))
+        except Exception as ex:
+            raise CmdException('header decoding error: %s' % str(ex))
 
     # parse the headers
-    if msg.has_key('from'):
+    if 'from' in msg:
         authname, authemail = name_email(__decode_header(msg['from']))
     else:
         authname = authemail = None
@@ -422,21 +447,22 @@ def parse_mail(msg):
 
     # remove the '[*PATCH*]' expression in the subject
     if descr:
-        descr = re.findall('^(\[.*?[Pp][Aa][Tt][Cc][Hh].*?\])?\s*(.*)$',
+        descr = re.findall(r'^(\[.*?[Pp][Aa][Tt][Cc][Hh].*?\])?\s*(.*)$',
                            descr)[0][1]
     else:
-        raise CmdException, 'Subject: line not found'
+        raise CmdException('Subject: line not found')
 
     # the rest of the message
-    msg_text = ''
+    msg_data = b''
     for part in msg.walk():
         if part.get_content_type() in ['text/plain',
                                        'application/octet-stream']:
-            msg_text += part.get_payload(decode = True)
+            payload = part.get_payload(decode=True)
+            msg_data += payload
 
-    rem_descr, diff = __split_descr_diff(msg_text)
+    rem_descr, diff = __split_descr_diff(msg_data)
     if rem_descr:
-        descr += '\n\n' + rem_descr
+        descr += '\n\n' + decode_utf8_with_latin1(rem_descr)
 
     # parse the description for author information
     descr, descr_authname, descr_authemail, descr_authdate = \
@@ -450,15 +476,20 @@ def parse_mail(msg):
 
     return (descr, authname, authemail, authdate, diff)
 
-def parse_patch(text, contains_diff):
-    """Parse the input text and return (description, authname,
-    authemail, authdate, diff)
+def parse_patch(patch_data, contains_diff):
+    """Parse patch data.
+
+    Returns (description, authname, authemail, authdate, diff)
+
     """
+    assert isinstance(patch_data, bytes)
     if contains_diff:
-        (text, diff) = __split_descr_diff(text)
+        (descr, diff) = __split_descr_diff(patch_data)
     else:
+        descr = patch_data
         diff = None
-    (descr, authname, authemail, authdate) = __parse_description(text)
+    (descr, authname, authemail, authdate) = __parse_description(
+        decode_utf8_with_latin1(descr))
 
     # we don't yet have an agreed place for the creation date.
     # Just return None
@@ -489,8 +520,8 @@ def run_commit_msg_hook(repo, cd, editor_is_used=True):
 
     try:
         new_msg = run_hook_on_string(commit_msg_hook, cd.message)
-    except RunException, exc:
-        raise EditorException, str(exc)
+    except RunException as exc:
+        raise EditorException(str(exc))
 
     return cd.set_message(new_msg)
 
@@ -505,11 +536,11 @@ def update_commit_data(cd, options):
     cd = cd.set_author(options.author(cd.author))
 
     # Add Signed-off-by: or similar.
-    if options.sign_str != None:
+    if options.sign_str is not None:
         sign_str = options.sign_str
     else:
         sign_str = config.get("stgit.autosign")
-    if sign_str != None:
+    if sign_str is not None:
         cd = cd.set_message(
             add_sign_line(cd.message, sign_str,
                           cd.committer.name, cd.committer.email))
@@ -517,6 +548,9 @@ def update_commit_data(cd, options):
     # Let user edit the commit message manually, unless
     # --save-template or --message was specified.
     if not getattr(options, 'save_template', None) and options.message is None:
+        tmpl = templates.get_template('patchdescr.tmpl')
+        if tmpl:
+            cd = cd.set_message(cd.message + tmpl)
         cd = cd.set_message(edit_string(cd.message, '.stgit-new.txt'))
 
     return cd
