@@ -1,3 +1,15 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse, git
+from stgit.argparse import opt
+from stgit.commands.common import (DirectoryHasRepository,
+                                   color_diff_flags,
+                                   git_id)
+from stgit.lib import git as gitlib
+from stgit.out import out
+from stgit.pager import pager
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -15,15 +27,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os
-from pydoc import pager
-from stgit.argparse import opt
-from stgit.commands.common import *
-from stgit.utils import *
-from stgit.out import *
-from stgit import argparse, stack, git
-from stgit.lib import git as gitlib
-
 help = 'Show the tree diff'
 kind = 'wc'
 usage = ['[options] [--] [<files or dirs>]']
@@ -44,7 +47,9 @@ options = [
         short = 'Show the stat instead of the diff'),
     ] + argparse.diff_opts_option()
 
-directory = DirectoryHasRepository(log = False)
+directory = DirectoryHasRepository(log=False)
+crt_series = None
+
 
 def func(parser, options, args):
     """Show the tree diff
@@ -69,11 +74,10 @@ def func(parser, options, args):
 
     if not options.stat:
         options.diff_flags.extend(color_diff_flags())
-    diff_str = git.diff(args, rev1 and git_id(crt_series, rev1),
-                        rev2 and git_id(crt_series, rev2),
-                        diff_flags = options.diff_flags)
+    diff = git.diff(args, rev1 and git_id(crt_series, rev1),
+                    rev2 and git_id(crt_series, rev2),
+                    diff_flags = options.diff_flags)
     if options.stat:
-        out.stdout_raw(gitlib.diffstat(diff_str) + '\n')
-    else:
-        if diff_str:
-            pager(diff_str)
+        out.stdout_raw(gitlib.diffstat(diff) + '\n')
+    elif diff:
+        pager(diff)
