@@ -1,5 +1,14 @@
-"""Patch editing command
-"""
+# -*- coding: utf-8 -*-
+"""Patch editing command"""
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import io
+
+from stgit import argparse, utils
+from stgit.argparse import opt
+from stgit.commands import common
+from stgit.lib import transaction, edit
+from stgit.out import out
 
 __copyright__ = """
 Copyright (C) 2007, Catalin Marinas <catalin.marinas@gmail.com>
@@ -17,12 +26,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-from stgit.argparse import opt
-from stgit import argparse, git, utils
-from stgit.commands import common
-from stgit.lib import git as gitlib, transaction, edit
-from stgit.out import *
-
 help = 'Edit a patch description or diff'
 kind = 'patch'
 usage = ['[options] [--] [<patch>]']
@@ -101,9 +104,13 @@ def func(parser, options, args):
                 options.set_tree, discard_stderr = True, object_type = 'tree'))
 
     cd, failed_diff = edit.auto_edit_patch(
-        stack.repository, cd, msg = options.message, contains_diff = True,
-        author = options.author, committer = lambda p: p,
-        sign_str = options.sign_str)
+        stack.repository, cd,
+        msg=(None if options.message is None else
+             options.message.encode('utf-8')),
+        contains_diff=True,
+        author=options.author,
+        committer=lambda p: p,
+        sign_str=options.sign_str)
 
     if options.save_template:
         options.save_template(
@@ -118,10 +125,10 @@ def func(parser, options, args):
 
     def failed(reason='Edited patch did not apply.'):
         fn = '.stgit-failed.patch'
-        f = file(fn, 'w')
-        f.write(edit.patch_desc(stack.repository, cd,
-                                options.diff, options.diff_flags, failed_diff))
-        f.close()
+        with io.open(fn, 'wb') as f:
+            f.write(edit.patch_desc(stack.repository, cd,
+                                    options.diff, options.diff_flags,
+                                    failed_diff))
         out.error(reason,
                   'The patch has been saved to "%s".' % fn)
         return utils.STGIT_COMMAND_ERROR
