@@ -1,5 +1,17 @@
-"""Export command
-"""
+# -*- coding: utf-8 -*-
+"""Export command"""
+
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import io
+import os
+import sys
+
+from stgit import argparse, git, templates
+from stgit.argparse import opt
+from stgit.commands import common
+from stgit.lib import git as gitlib
+from stgit.out import out
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -17,14 +29,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import os
-import sys
-from stgit.argparse import opt
-from stgit.commands import common
-from stgit import argparse, git, templates
-from stgit.out import out
-from stgit.lib import git as gitlib
-
 help = 'Export patches to a directory'
 kind = 'patch'
 usage = ['[options] [--] [<patch1>] [<patch2>] [<patch3>..<patch4>]']
@@ -84,11 +88,6 @@ def func(parser, options, args):
         out.warn('Local changes in the tree;'
                  ' you might want to commit them first')
 
-    if not options.stdout:
-        if not os.path.isdir(dirname):
-            os.makedirs(dirname)
-        series = file(os.path.join(dirname, 'series'), 'w+')
-
     applied = stack.patchorder.applied
     unapplied = stack.patchorder.unapplied
     if len(args) != 0:
@@ -98,7 +97,7 @@ def func(parser, options, args):
 
     num = len(patches)
     if num == 0:
-        raise common.CmdException, 'No patches applied'
+        raise common.CmdException('No patches applied')
 
     zpadding = len(str(num))
     if zpadding < 2:
@@ -106,19 +105,22 @@ def func(parser, options, args):
 
     # get the template
     if options.template:
-        tmpl = file(options.template).read()
+        with io.open(options.template, 'r') as f:
+            tmpl = f.read()
     else:
         tmpl = templates.get_template('patchexport.tmpl')
         if not tmpl:
             tmpl = ''
 
-    # note the base commit for this series
     if not options.stdout:
+        if not os.path.isdir(dirname):
+            os.makedirs(dirname)
+        series = io.open(os.path.join(dirname, 'series'), 'w')
+        # note the base commit for this series
         base_commit = stack.base.sha1
-        print >> series, '# This series applies on GIT commit %s' % base_commit
+        print('# This series applies on GIT commit %s' % base_commit, file=series)
 
-    patch_no = 1;
-    for p in patches:
+    for patch_no, p in enumerate(patches, 1):
         pname = p
         if options.patch:
             pname = '%s.patch' % pname
@@ -128,7 +130,7 @@ def func(parser, options, args):
             pname = '%s-%s' % (str(patch_no).zfill(zpadding), pname)
         pfile = os.path.join(dirname, pname)
         if not options.stdout:
-            print >> series, pname
+            print(pname, file=series)
 
         # get the patch description
         patch = stack.patches.get(p)
@@ -138,8 +140,7 @@ def func(parser, options, args):
         descr_lines = descr.split('\n')
 
         short_descr = descr_lines[0].rstrip()
-        long_descr = reduce(lambda x, y: x + '\n' + y,
-                            descr_lines[1:], '').strip()
+        long_descr = '\n'.join(descr_lines[1:]).strip()
 
         diff = stack.repository.diff_tree(cd.parent.data.tree, cd.tree, options.diff_flags)
 
@@ -152,34 +153,34 @@ def func(parser, options, args):
                      'authdate': cd.author.date.isoformat(),
                      'commname': cd.committer.name,
                      'commemail': cd.committer.email}
-        for key in tmpl_dict:
-            if not tmpl_dict[key]:
-                tmpl_dict[key] = ''
 
         try:
-            descr = tmpl % tmpl_dict
-        except KeyError, err:
-            raise common.CmdException, 'Unknown patch template variable: %s' \
-                  % err
+            descr = templates.specialize_template(tmpl, tmpl_dict)
+        except KeyError as err:
+            raise common.CmdException('Unknown patch template variable: %s' %
+                                      err)
         except TypeError:
-            raise common.CmdException, 'Only "%(name)s" variables are ' \
-                  'supported in the patch template'
+            raise common.CmdException('Only "%(name)s" variables are '
+                                      'supported in the patch template')
 
         if options.stdout:
-            f = sys.stdout
+            if hasattr(sys.stdout, 'buffer'):
+                f = sys.stdout.buffer
+            else:
+                f = sys.stdout
         else:
-            f = open(pfile, 'w+')
+            f = io.open(pfile, 'wb')
 
         if options.stdout and num > 1:
-            print '-'*79
-            print patch.name
-            print '-'*79
+            f.write('\n'.join(['-' * 79,
+                               patch.name,
+                               '-' * 79,
+                               '']).encode('utf-8'))
 
         f.write(descr)
         f.write(diff)
         if not options.stdout:
             f.close()
-        patch_no += 1
 
     if not options.stdout:
         series.close()
