@@ -1,3 +1,15 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import io
+import re
+import sys
+
+from stgit import argparse
+from stgit.argparse import opt
+from stgit.commands import common
+from stgit.lib import transaction
+
 __copyright__ = """
 Copyright (C) 2006, Robin Rosenberg <robin.rosenberg@dewire.com>
 Modified by Catalin Marinas
@@ -15,13 +27,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import re
-import sys
-from stgit.argparse import opt
-from stgit.commands import common
-from stgit.lib import transaction
-from stgit import argparse
-
 help = 'Push patches to the top, even if applied'
 kind = 'stack'
 usage = ['[--] <patches>',
@@ -53,9 +58,9 @@ def func(parser, options, args):
 
     if options.series:
         if options.series == '-':
-            f = sys.stdin
+            f = io.open(sys.stdin.fileno())
         else:
-            f = file(options.series)
+            f = io.open(options.series)
 
         patches = []
         for line in f:
@@ -70,7 +75,7 @@ def func(parser, options, args):
 
     applied = [p for p in stack.patchorder.applied if p not in patches] + \
             patches
-    unapplied = [p for p in stack.patchorder.unapplied if not p in patches]
+    unapplied = [p for p in stack.patchorder.unapplied if p not in patches]
 
     iw = stack.repository.default_iw
     clean_iw = (not options.keep and iw) or None
