@@ -1,3 +1,11 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse
+from stgit.commands import common
+from stgit.lib import transaction
+
 __copyright__ = """
 Copyright (C) 2006, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,11 +22,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-from stgit.commands import common
-from stgit.lib import transaction
-from stgit import argparse
-from stgit.argparse import opt
-
 help = 'Push or pop patches to the given one'
 kind = 'stack'
 usage = ['[options] [--] <patch-name>']
@@ -42,7 +45,7 @@ def func(parser, options, args):
     trans = transaction.StackTransaction(stack, 'goto',
                                          check_clean_iw = clean_iw)
 
-    if not patch in trans.all_patches:
+    if patch not in trans.all_patches:
         candidate = common.get_patch_from_list(patch, trans.all_patches)
         if candidate is None:
             raise common.CmdException('Patch "%s" does not exist' % patch)
