@@ -1,3 +1,12 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse
+from stgit.commands import common
+from stgit.lib import transaction
+from stgit.out import out
+
 __copyright__ = """
 Copyright (C) 2009, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,11 +23,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-from stgit.commands import common
-from stgit.lib import transaction
-from stgit import argparse, out
-from stgit.argparse import opt
-
 help = 'Hide a patch in the series'
 kind = 'stack'
 usage = ['[options] [--] <patch-range>']
@@ -29,8 +33,8 @@ the plain 'series' command output."""
 args = [argparse.patch_range(argparse.applied_patches,
                              argparse.unapplied_patches)]
 options = [
-    opt('-b', '--branch', args = [argparse.stg_branches],
-        short = 'Use BRANCH instead of the default branch')]
+    argparse.opt('-b', '--branch', args=[argparse.stg_branches],
+                 short='Use BRANCH instead of the default branch')]
 
 directory = common.DirectoryHasRepositoryLib()
 
@@ -46,10 +50,10 @@ def func(parser, options, args):
     for p in patches:
         if p in trans.hidden:
             out.warn('Patch "%s" already hidden' % p)
-    patches = [p for p in patches if not p in set(trans.hidden)]
+    patches = [p for p in patches if p not in set(trans.hidden)]
 
-    applied = [p for p in trans.applied if not p in set(patches)]
-    unapplied = [p for p in trans.unapplied if not p in set(patches)]
+    applied = [p for p in trans.applied if p not in set(patches)]
+    unapplied = [p for p in trans.unapplied if p not in set(patches)]
     hidden = patches + trans.hidden
 
     trans.reorder_patches(applied, unapplied, hidden)
