@@ -1,3 +1,32 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+from contextlib import closing
+import bz2
+import gzip
+import mailbox
+import os
+import re
+import sys
+import tarfile
+
+from stgit import argparse, git
+from stgit.argparse import opt
+from stgit.compat import message_from_binary_file, text
+from stgit.config import config
+from stgit.lib import git as gitlib
+from stgit.out import out
+from stgit.commands.common import (CmdException,
+                                   DirectoryHasRepository,
+                                   check_conflicts,
+                                   check_head_top_equal,
+                                   check_local_changes,
+                                   git_id,
+                                   parse_mail,
+                                   parse_patch,
+                                   print_crt_patch)
+from stgit.utils import make_patch_name
+
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,15 +43,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os, re, email, tarfile
-from mailbox import UnixMailbox
-from StringIO import StringIO
-from stgit.argparse import opt
-from stgit.commands.common import *
-from stgit.utils import *
-from stgit.out import *
-from stgit import argparse, stack, git
-
 name = 'import'
 help = 'Import a GNU diff file as a new patch'
 kind = 'patch'
@@ -72,21 +92,15 @@ options = [
         short = 'Invoke an editor for the patch description'),
     opt('-d', '--showdiff', action = 'store_true',
         short = 'Show the patch content in the editor buffer'),
-    opt('-a', '--author', metavar = '"NAME <EMAIL>"',
-        short = 'Use "NAME <EMAIL>" as the author details'),
-    opt('--authname',
-        short = 'Use AUTHNAME as the author name'),
-    opt('--authemail',
-        short = 'Use AUTHEMAIL as the author e-mail'),
-    opt('--authdate',
-        short = 'Use AUTHDATE as the author date'),
-    ] + argparse.sign_options()
+    ] + argparse.author_options() + argparse.sign_options()
+
+directory = DirectoryHasRepository(log=True)
+crt_series = None
 
-directory = DirectoryHasRepository(log = True)
 
 def __strip_patch_name(name):
-    stripped = re.sub('^[0-9]+-(.*)$', '\g<1>', name)
-    stripped = re.sub('^(.*)\.(diff|patch)$', '\g<1>', stripped)
+    stripped = re.sub('^[0-9]+-(.*)$', r'\g<1>', name)
+    stripped = re.sub(r'^(.*)\.(diff|patch)$', r'\g<1>', stripped)
 
     return stripped
 
@@ -116,7 +130,7 @@ def __create_patch(filename, message, author_name, author_email,
         patch = make_patch_name(message, unacceptable_name)
     else:
         # fix possible invalid characters in the patch name
-        patch = re.sub('[^\w.]+', '-', patch).strip('-')
+        patch = re.sub(r'[^\w.]+', '-', patch).strip('-')
 
     if options.ignore and patch in crt_series.get_applied():
         out.info('Ignoring already applied patch "%s"' % patch)
@@ -124,21 +138,14 @@ def __create_patch(filename, message, author_name, author_email,
     if options.replace and patch in crt_series.get_unapplied():
         crt_series.delete_patch(patch, keep_log = True)
 
-    # refresh_patch() will invoke the editor in this case, with correct
-    # patch content
-    if not message:
-        can_edit = False
-
-    if options.author:
-        options.authname, options.authemail = name_email(options.author)
-
     # override the automatically parsed settings
-    if options.authname:
-        author_name = options.authname
-    if options.authemail:
-        author_email = options.authemail
-    if options.authdate:
-        author_date = options.authdate
+    author = options.author(gitlib.Person())
+    if author.name:
+        author_name = author.name
+    if author.email:
+        author_email = author.email
+    if author.date:
+        author_date = text(author.date)
 
     sign_str = options.sign_str
     if not options.sign_str:
@@ -179,18 +186,17 @@ def __get_handle_and_name(filename):
     """Return a file object and a patch name derived from filename
     """
     # see if it's a gzip'ed or bzip2'ed patch
-    import bz2, gzip
     for copen, ext in [(gzip.open, '.gz'), (bz2.BZ2File, '.bz2')]:
         try:
             f = copen(filename)
             f.read(1)
             f.seek(0)
             return (f, __mkpatchname(filename, ext))
-        except IOError, e:
+        except IOError:
             pass
 
     # plain old file...
-    return (open(filename), filename)
+    return (open(filename, 'rb'), filename)
 
 def __import_file(filename, options, patch = None):
     """Import a patch from a file or standard input
@@ -199,7 +205,7 @@ def __import_file(filename, options, patch = None):
     if filename:
         (f, pname) = __get_handle_and_name(filename)
     else:
-        f = sys.stdin
+        f = os.fdopen(sys.__stdin__.fileno(), 'rb')
 
     if patch:
         pname = patch
@@ -208,14 +214,15 @@ def __import_file(filename, options, patch = None):
 
     if options.mail:
         try:
-            msg = email.message_from_file(f)
-        except Exception, ex:
-            raise CmdException, 'error parsing the e-mail file: %s' % str(ex)
+            msg = message_from_binary_file(f)
+        except Exception as ex:
+            raise CmdException('error parsing the e-mail file: %s' % str(ex))
         message, author_name, author_email, author_date, diff = \
                  parse_mail(msg)
     else:
+        patch_str = f.read()
         message, author_name, author_email, author_date, diff = \
-                 parse_patch(f.read(), contains_diff = True)
+                 parse_patch(patch_str, contains_diff=True)
 
     if filename:
         f.close()
@@ -226,13 +233,11 @@ def __import_file(filename, options, patch = None):
 def __import_series(filename, options):
     """Import a series of patches
     """
-    applied = crt_series.get_applied()
-
     if filename:
         if tarfile.is_tarfile(filename):
             __import_tarfile(filename, options)
             return
-        f = file(filename)
+        f = open(filename)
         patchdir = os.path.dirname(filename)
     else:
         f = sys.stdin
@@ -250,10 +255,12 @@ def __import_series(filename, options):
         if m:
             patch = m.group('patchfilename')
             if m.group('striplevel') != '0':
-                raise CmdException, "error importing quilt series, patch '%s' has unsupported strip level: '-p%s'" % (patch, m.group('striplevel'))
+                raise CmdException("error importing quilt series, patch '%s'"
+                                   " has unsupported strip level: '-p%s'" %
+                                   (patch, m.group('striplevel')))
             options.strip = 0
         patchfile = os.path.join(patchdir, patch)
-        patch = __replace_slashes_with_dashes(patch);
+        patch = __replace_slashes_with_dashes(patch)
 
         __import_file(patchfile, options, patch)
 
@@ -264,35 +271,52 @@ def __import_mbox(filename, options):
     """Import a series from an mbox file
     """
     if filename:
-        f = file(filename, 'rb')
+        namedtemp = None
     else:
-        f = StringIO(sys.stdin.read())
+        from tempfile import NamedTemporaryFile
+        stdin = os.fdopen(sys.__stdin__.fileno(), 'rb')
+        namedtemp = NamedTemporaryFile('wb', suffix='.mbox', delete=False)
+        namedtemp.write(stdin.read())
+        namedtemp.close()
+        filename = namedtemp.name
 
     try:
-        mbox = UnixMailbox(f, email.message_from_file)
-    except Exception, ex:
-        raise CmdException, 'error parsing the mbox file: %s' % str(ex)
+        try:
+            mbox = mailbox.mbox(filename, message_from_binary_file,
+                                create=False)
+        except Exception as ex:
+            raise CmdException('error parsing the mbox file: %s' % str(ex))
 
-    for msg in mbox:
-        message, author_name, author_email, author_date, diff = \
-                 parse_mail(msg)
-        __create_patch(None, message, author_name, author_email,
-                       author_date, diff, options)
+        with closing(mbox):
+            for msg in mbox:
+                (message,
+                 author_name,
+                 author_email,
+                 author_date,
+                 diff) = parse_mail(msg)
+                __create_patch(None, message, author_name, author_email,
+                               author_date, diff, options)
+    finally:
+        if namedtemp is not None:
+            os.unlink(namedtemp.name)
 
-    f.close()
 
 def __import_url(url, options):
     """Import a patch from a URL
     """
-    import urllib
+    try:
+        from urllib.request import urlretrieve
+        from urllib.parse import unquote
+    except ImportError:
+        from urllib import urlretrieve, unquote
     import tempfile
 
     if not url:
         raise CmdException('URL argument required')
 
-    patch = os.path.basename(urllib.unquote(url))
+    patch = os.path.basename(unquote(url))
     filename = os.path.join(tempfile.gettempdir(), patch)
-    urllib.urlretrieve(url, filename)
+    urlretrieve(url, filename)
     __import_file(filename, options)
 
 def __import_tarfile(tar, options):
@@ -302,7 +326,7 @@ def __import_tarfile(tar, options):
     import shutil
 
     if not tarfile.is_tarfile(tar):
-        raise CmdException, "%s is not a tarfile!" % tar
+        raise CmdException("%s is not a tarfile!" % tar)
 
     t = tarfile.open(tar, 'r')
     names = t.getnames()
@@ -310,18 +334,18 @@ def __import_tarfile(tar, options):
     # verify paths in the tarfile are safe
     for n in names:
         if n.startswith('/'):
-            raise CmdException, "Absolute path found in %s" % tar
+            raise CmdException("Absolute path found in %s" % tar)
         if n.find("..") > -1:
-            raise CmdException, "Relative path found in %s" % tar
+            raise CmdException("Relative path found in %s" % tar)
 
     # find the series file
-    seriesfile = '';
+    seriesfile = ''
     for m in names:
         if m.endswith('/series') or m == 'series':
             seriesfile = m
             break
     if seriesfile == '':
-        raise CmdException, "no 'series' file found in %s" % tar
+        raise CmdException("no 'series' file found in %s" % tar)
 
     # unpack into a tmp dir
     tmpdir = tempfile.mkdtemp('.stg')
