@@ -1,3 +1,9 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit.commands import common
+from stgit.lib import stack
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -15,9 +21,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-from stgit.commands import common
-from stgit.lib import stack
-
 help = 'Initialise the current branch for use with StGIT'
 kind = 'stack'
 usage = ['']
