@@ -1,3 +1,12 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse
+from stgit.argparse import opt
+from stgit.commands import common
+from stgit.out import out
+
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,11 +23,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-from stgit.argparse import opt
-from stgit.commands import common
-from stgit.out import out
-from stgit import argparse
-
 help = 'Print the name of the next patch'
 kind = 'stack'
 usage = ['']
@@ -44,4 +48,4 @@ def func(parser, options, args):
     if unapplied:
         out.stdout(unapplied[0])
     else:
-        raise common.CmdException, 'No unapplied patches'
+        raise common.CmdException('No unapplied patches')
