@@ -1,3 +1,13 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse, git
+from stgit.argparse import opt
+from stgit.commands.common import CmdException, DirectoryHasRepository
+from stgit.out import out
+from stgit.pager import pager
+
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,14 +24,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os
-from pydoc import pager
-from stgit.argparse import opt
-from stgit.commands.common import *
-from stgit.utils import *
-from stgit.out import *
-from stgit import argparse, stack, git
-
 help = 'Show the applied patches modifying a file'
 kind = 'stack'
 usage = ['[options] [--] [<files or dirs>]']
@@ -38,7 +40,8 @@ options = [
     opt('-b', '--branch', args = [argparse.stg_branches],
         short = 'Use BRANCH instead of the default branch')]
 
-directory = DirectoryHasRepository(log = False)
+directory = DirectoryHasRepository(log=False)
+crt_series = None
 
 diff_tmpl = \
           '-------------------------------------------------------------------------------\n' \
@@ -59,11 +62,11 @@ def func(parser, options, args):
     directory.cd_to_topdir()
 
     if not files:
-        raise CmdException, 'No files specified or no local changes'
+        raise CmdException('No files specified or no local changes')
 
     applied = crt_series.get_applied()
     if not applied:
-        raise CmdException, 'No patches applied'
+        raise CmdException('No patches applied')
 
     revs = git.modifying_revs(files, crt_series.get_base(),
                               crt_series.get_head())
@@ -89,4 +92,4 @@ def func(parser, options, args):
                 out.stdout(patch.get_name())
 
     if options.diff:
-        pager(diff_output)
+        pager(diff_output.encode('utf-8'))
