@@ -1,3 +1,23 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse, git
+from stgit.argparse import opt
+from stgit.commands.common import (CmdException,
+                                   DirectoryGotoToplevel,
+                                   check_head_top_equal,
+                                   check_local_changes,
+                                   check_conflicts,
+                                   git_id,
+                                   name_email_date,
+                                   parse_patches,
+                                   parse_rev,
+                                   print_crt_patch)
+from stgit.out import out
+from stgit.stack import Series
+from stgit.utils import find_patch_name
+
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,14 +34,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os
-from stgit.argparse import opt
-from stgit.commands.common import *
-from stgit.utils import *
-from stgit.out import *
-from stgit import argparse, stack, git
-from stgit.stack import Series
-
 help = 'Import a patch from a different branch or a commit object'
 kind = 'patch'
 usage = ['[options] [--] ([<patch1>] [<patch2>] [<patch3>..<patch4>])|<commit>']
@@ -56,7 +68,9 @@ options = [
     opt('--unapplied', action = 'store_true',
         short = 'Keep the patch unapplied')]
 
-directory = DirectoryGotoToplevel(log = True)
+directory = DirectoryGotoToplevel(log=True)
+crt_series = None
+
 
 def __pick_commit(commit_id, patchname, options):
     """Pick a commit id.
@@ -101,7 +115,7 @@ def __pick_commit(commit_id, patchname, options):
         out.start('Updating with commit %s' % commit_id)
 
         if not git.apply_diff(bottom, top, files = files):
-            raise CmdException, 'Patch updating failed'
+            raise CmdException('Patch updating failed')
 
         out.done()
     else:
@@ -198,12 +212,12 @@ def func(parser, options, args):
 
     if not commit_id and len(patches) > 1:
         if options.name:
-            raise CmdException, '--name can only be specified with one patch'
+            raise CmdException('--name can only be specified with one patch')
         if options.parent:
-            raise CmdException, '--parent can only be specified with one patch'
+            raise CmdException('--parent can only be specified with one patch')
 
     if options.update and not crt_series.get_current():
-        raise CmdException, 'No patches applied'
+        raise CmdException('No patches applied')
 
     if commit_id:
         # Try to guess a patch name if the argument was <branch>:<patch>
