@@ -1,3 +1,11 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse
+from stgit.argparse import opt
+from stgit.commands import common
+from stgit.lib import transaction
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -15,11 +23,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-from stgit.commands import common
-from stgit.lib import transaction
-from stgit import argparse
-from stgit.argparse import opt
-
 help = 'Push one or more patches onto the stack'
 kind = 'stack'
 usage = ['[options] [--] [<patch1>] [<patch2>] [<patch3>..<patch4>]']
@@ -83,10 +86,19 @@ def func(parser, options, args):
     elif not args:
         patches = [trans.unapplied[0]]
     else:
-        patches = common.parse_patches(args, trans.unapplied)
+        try:
+            patches = common.parse_patches(args, trans.unapplied)
+        except common.CmdException as e:
+            try:
+                patches = common.parse_patches(args, trans.applied)
+            except common.CmdException:
+                raise e
+            else:
+                raise common.CmdException('Patch%s already applied: %s' % (
+                    'es' if len(patches) > 1 else '',
+                    ', '.join(patches)))
 
-    if not patches:
-        raise common.CmdException('No patches to push')
+    assert patches
 
     if options.reverse:
         patches.reverse()
