@@ -1,3 +1,21 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse
+from stgit.argparse import opt
+from stgit.commands.common import (CmdException,
+                                   DirectoryGotoToplevel,
+                                   check_conflicts,
+                                   check_head_top_equal,
+                                   check_local_changes,
+                                   git_id,
+                                   post_rebase,
+                                   prepare_rebase,
+                                   print_crt_patch,
+                                   rebase)
+from stgit.git import GitException
+
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,12 +32,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os
-from stgit.argparse import opt
-from stgit.commands.common import *
-from stgit.utils import *
-from stgit import argparse, stack, git
-
 help = 'Move the stack base to another point in history'
 kind = 'stack'
 usage = ['[options] [--] <new-base-id>']
@@ -46,7 +58,9 @@ options = [
     opt('-m', '--merged', action = 'store_true',
         short = 'Check for patches merged upstream')]
 
-directory = DirectoryGotoToplevel(log = True)
+directory = DirectoryGotoToplevel(log=True)
+crt_series = None
+
 
 def func(parser, options, args):
     """Rebase the current stack
@@ -55,15 +69,15 @@ def func(parser, options, args):
         parser.error('incorrect number of arguments')
 
     if crt_series.get_protected():
-        raise CmdException, 'This branch is protected. Rebase is not permitted'
+        raise CmdException('This branch is protected. Rebase is not permitted')
 
     check_local_changes()
     check_conflicts()
     check_head_top_equal(crt_series)
 
     # ensure an exception is raised before popping on non-existent target
-    if git_id(crt_series, args[0]) == None:
-        raise GitException, 'Unknown revision: %s' % args[0]
+    if git_id(crt_series, args[0]) is None:
+        raise GitException('Unknown revision: %s' % args[0])
         
     applied = prepare_rebase(crt_series)
     rebase(crt_series, args[0])
