@@ -1,4 +1,15 @@
 # -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import re
+
+from stgit import git
+from stgit.commands.common import (CmdException,
+                                   DirectoryGotoToplevel,
+                                   name_email_date)
+from stgit.out import out
+from stgit.run import Run
+from stgit.utils import make_patch_name
 
 __copyright__ = """
 Copyright (C) 2006, Karl Hasselström <kha@treskal.com>
@@ -16,14 +27,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os
-from stgit.argparse import opt
-from stgit.commands.common import *
-from stgit.utils import *
-from stgit.out import *
-from stgit.run import *
-from stgit import stack, git
-
 help = 'Fix StGit metadata if branch was modified with git commands'
 kind = 'stack'
 usage = ['']
@@ -71,7 +74,9 @@ repair" is _not_ what you want. In that case, what you want is option
 args = []
 options = []
 
-directory = DirectoryGotoToplevel(log = True)
+directory = DirectoryGotoToplevel(log=True)
+crt_series = None
+
 
 class Commit(object):
     def __init__(self, id):
@@ -100,7 +105,7 @@ def read_commit_dag(branch):
     for line in Run('git', 'rev-list', '--parents', '--all').output_lines():
         cs = line.split()
         for id in cs:
-            if not id in commits:
+            if id not in commits:
                 commits[id] = Commit(id)
         for id in cs[1:]:
             commits[cs[0]].parents.add(commits[id])
@@ -207,12 +212,13 @@ def func(parser, options, args):
         out.info('%s is now unapplied' % name)
     for name in hidden_name_set - orig_hidden_name_set:
         out.info('%s is now hidden' % name)
-    orig_order = dict(zip(orig_patches, xrange(len(orig_patches))))
-    def patchname_cmp(p1, p2):
-        i1 = orig_order.get(p1, len(orig_order))
-        i2 = orig_order.get(p2, len(orig_order))
-        return cmp((i1, p1), (i2, p2))
+    orig_order = dict(zip(orig_patches, range(len(orig_patches))))
+
+    def patchname_key(p):
+        i = orig_order.get(p, len(orig_order))
+        return i, p
+
     crt_series.set_applied(p.patch for p in applied)
-    crt_series.set_unapplied(sorted(unapplied_name_set, cmp = patchname_cmp))
-    crt_series.set_hidden(sorted(hidden_name_set, cmp = patchname_cmp))
+    crt_series.set_unapplied(sorted(unapplied_name_set, key=patchname_key))
+    crt_series.set_hidden(sorted(hidden_name_set, key=patchname_key))
     out.done()
