@@ -1,3 +1,13 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse
+from stgit.argparse import opt
+from stgit.commands import common
+from stgit.commands.common import parse_patches
+from stgit.config import config
+from stgit.out import out
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -15,13 +25,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-from stgit.argparse import opt
-from stgit.commands import common
-from stgit.commands.common import parse_patches
-from stgit.out import out
-from stgit.config import config
-from stgit import argparse
-
 help = 'Print the patch series'
 kind = 'stack'
 usage = ['[options] [--] [<patch-range>]']
@@ -141,7 +144,7 @@ def func(parser, options, args):
     """Show the patch series
     """
     if options.all and options.short:
-        raise common.CmdException, 'combining --all and --short is meaningless'
+        raise common.CmdException('combining --all and --short is meaningless')
 
     stack = directory.repository.get_stack(options.branch)
     if options.missing:
@@ -152,8 +155,8 @@ def func(parser, options, args):
     applied = unapplied = hidden = ()
     if options.applied or options.unapplied or options.hidden:
         if options.all:
-            raise common.CmdException, \
-                '--all cannot be used with --applied/unapplied/hidden'
+            raise common.CmdException('--all cannot be used with'
+                                      ' --applied/unapplied/hidden')
         if options.applied:
             applied = stack.patchorder.applied
         if options.unapplied:
