@@ -1,3 +1,21 @@
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import os
+import re
+
+from stgit import argparse, stack, git
+from stgit.argparse import opt
+from stgit.commands.common import (CmdException,
+                                   DirectoryGotoToplevel,
+                                   check_local_changes,
+                                   check_conflicts,
+                                   check_head_top_equal,
+                                   parse_patches,
+                                   pop_patches,
+                                   push_patches)
+from stgit.out import out
+
 __copyright__ = """
 Copyright (C) 2006, Catalin Marinas <catalin.marinas@gmail.com>
 
@@ -14,14 +32,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os
-import stgit.commands.common
-from stgit.argparse import opt
-from stgit.commands.common import *
-from stgit.utils import *
-from stgit.out import *
-from stgit import argparse, stack, git
-
 help = 'Synchronise patches with a branch or a series'
 kind = 'patch'
 usage = ['[options] [--] [<patch1>] [<patch2>] [<patch3>..<patch4>]']
@@ -42,7 +52,9 @@ options = [
     opt('-s', '--series', args = [argparse.files],
         short = 'Syncronise patches with SERIES')]
 
-directory = DirectoryGotoToplevel(log = True)
+directory = DirectoryGotoToplevel(log=True)
+crt_series = None
+
 
 def __check_all():
     check_local_changes()
@@ -67,7 +79,7 @@ def func(parser, options, args):
     if options.ref_branch:
         remote_series = stack.Series(options.ref_branch)
         if options.ref_branch == crt_series.get_name():
-            raise CmdException, 'Cannot synchronise with the current branch'
+            raise CmdException('Cannot synchronise with the current branch')
         remote_patches = remote_series.get_applied()
 
         # the merge function merge_patch(patch, pname)
@@ -77,19 +89,18 @@ def func(parser, options, args):
         patchdir = os.path.dirname(options.series)
 
         remote_patches = []
-        f = file(options.series)
-        for line in f:
-            p = re.sub('#.*$', '', line).strip()
-            if not p:
-                continue
-            remote_patches.append(p)
-        f.close()
+        with open(options.series) as f:
+            for line in f:
+                p = re.sub('#.*$', '', line).strip()
+                if not p:
+                    continue
+                remote_patches.append(p)
 
         # the merge function merge_patch(patch, pname)
         merge_patch = lambda patch, pname: \
                       __series_merge_patch(patch.get_bottom(), patchdir, pname)
     else:
-        raise CmdException, 'No remote branch or series specified'
+        raise CmdException('No remote branch or series specified')
 
     applied = crt_series.get_applied()
     unapplied = crt_series.get_unapplied()
@@ -105,14 +116,14 @@ def func(parser, options, args):
         parser.error('no patches applied')
 
     if not patches:
-        raise CmdException, 'No patches to synchronise'
+        raise CmdException('No patches to synchronise')
 
     __check_all()
 
     # only keep the patches to be synchronised
     sync_patches = [p for p in patches if p in remote_patches]
     if not sync_patches:
-        raise CmdException, 'No common patches to be synchronised'
+        raise CmdException('No common patches to be synchronised')
 
     # pop to the one before the first patch to be synchronised
     first_patch = sync_patches[0]
@@ -138,7 +149,6 @@ def func(parser, options, args):
         out.start('Synchronising "%s"' % p)
 
         patch = crt_series.get_patch(p)
-        bottom = patch.get_bottom()
         top = patch.get_top()
 
         # reset the patch backup information.
