@@ -1,4 +1,12 @@
 # -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from stgit import argparse, utils
+from stgit.argparse import opt
+from stgit.commands import common
+from stgit.lib import transaction
+from stgit.out import out
 
 __copyright__ = """
 Copyright (C) 2006, Karl Hasselström <kha@treskal.com>
@@ -16,12 +24,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-from stgit.argparse import opt
-from stgit.commands import common
-from stgit.lib import transaction
-from stgit.out import *
-from stgit import argparse, utils
-
 help = 'Turn regular git commits into StGit patches'
 kind = 'stack'
 usage = ['[--] <patch-name-1> [<patch-name-2> ...]',
@@ -72,7 +74,7 @@ def func(parser, options, args):
         to_commit = stack.repository.rev_parse(options.to)
         # check whether the --to commit is on a different branch
         merge_bases = directory.repository.get_merge_bases(to_commit, stack.base)
-        if not to_commit in merge_bases:
+        if to_commit not in merge_bases:
             to_commit = merge_bases[0]
             options.exclusive = True
     elif options.number:
@@ -84,7 +86,7 @@ def func(parser, options, args):
         elif len(args) == 1:
             # prefix specified
             patchnames = ['%s%d' % (args[0], i)
-                          for i in xrange(patch_nr, 0, -1)]
+                          for i in range(patch_nr, 0, -1)]
         else:
             parser.error('when using --number, specify at most one patch name')
     elif len(args) == 0:
@@ -95,7 +97,7 @@ def func(parser, options, args):
         patch_nr = len(patchnames)
 
     def check_and_append(c, n):
-        next = n.data.parents;
+        next = n.data.parents
         try:
             [next] = next
         except ValueError:
@@ -109,7 +111,7 @@ def func(parser, options, args):
     next_commit = stack.base
     if patch_nr:
         out.start('Uncommitting %d patches' % patch_nr)
-        for i in xrange(patch_nr):
+        for i in range(patch_nr):
             check_and_append(commits, next_commit)
             next_commit = next_commit.data.parent
     else:
