@@ -1,5 +1,13 @@
-"""Handles the Stacked GIT configuration files
-"""
+# -*- coding: utf-8 -*-
+"""Handles the Stacked GIT configuration files"""
+
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import os
+import re
+
+from stgit.exception import StgException
+from stgit.run import Run
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -17,31 +25,29 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import os, re
-from stgit import basedir
-from stgit.exception import *
-from stgit.run import *
 
 class GitConfigException(StgException):
     pass
 
-class GitConfig:
-    __defaults={
-        'stgit.smtpserver':     ['localhost:25'],
-        'stgit.smtpdelay':      ['5'],
-        'stgit.pullcmd':        ['git pull'],
-        'stgit.fetchcmd':       ['git fetch'],
-        'stgit.pull-policy':    ['pull'],
-        'stgit.autoimerge':     ['no'],
-        'stgit.keepoptimized':  ['no'],
-        'stgit.shortnr':        ['5'],
-        'stgit.pager':          ['less'],
-        'stgit.alias.add':      ['git add'],
-        'stgit.alias.rm':       ['git rm'],
-        'stgit.alias.mv':       ['git mv'],
-        'stgit.alias.resolved': ['git add'],
-        'stgit.alias.status':   ['git status -s']
-        }
+
+class GitConfig(object):
+    __defaults = {
+        'stgit.smtpserver':        ['localhost:25'],
+        'stgit.smtpdelay':         ['5'],
+        'stgit.pullcmd':           ['git pull'],
+        'stgit.fetchcmd':          ['git fetch'],
+        'stgit.pull-policy':       ['pull'],
+        'stgit.autoimerge':        ['no'],
+        'stgit.keepoptimized':     ['no'],
+        'stgit.refreshsubmodules': ['no'],
+        'stgit.shortnr':           ['5'],
+        'stgit.pager':             ['less'],
+        'stgit.alias.add':         ['git add'],
+        'stgit.alias.rm':          ['git rm'],
+        'stgit.alias.mv':          ['git mv'],
+        'stgit.alias.resolved':    ['git add'],
+        'stgit.alias.status':      ['git status -s']
+    }
 
     __cache = None
 
@@ -52,9 +58,13 @@ class GitConfig:
             return
         self.__cache = self.__defaults
         lines = Run('git', 'config', '--null', '--list'
-                   ).discard_exitcode().raw_output()
-        for line in filter(None, lines.split('\0')):
-            key, value = line.split('\n', 1)
+                    ).discard_exitcode().output_lines('\0')
+        for line in lines:
+            try:
+                key, value = line.split('\n', 1)
+            except ValueError:
+                key = line
+                value = None
             self.__cache.setdefault(key, []).append(value)
 
     def get(self, name):
@@ -73,16 +83,41 @@ class GitConfig:
 
     def getint(self, name):
         value = self.get(name)
-        if value == None:
+        if value is None:
             return None
         elif value.isdigit():
             return int(value)
         else:
-            raise GitConfigException, 'Value for "%s" is not an integer: "%s"' % (name, value)
+            raise GitConfigException('Value for "%s" is not an integer: "%s"' %
+                                     (name, value))
+
+    def getbool(self, name):
+        """Report the canonicalized boolean value for a given key."""
+        # We cannot directly call get() because we need to use the KeyError in
+        # order to distinguish between the case of a key with an undefined
+        # value, and a completely undefined key. Git expects the former to be
+        # reported as "true".
+        self.load()
+        try:
+            value = self.__cache[name][-1]
+        except KeyError:
+            return None
+        if value is None:
+            # The key is defined, but the value is not, so treat it as true.
+            return True
+        elif value in ['yes', 'on', 'true']:
+            return True
+        elif value in ['no', 'off', 'false', '']:
+            return False
+        elif value.isdigit():
+            return bool(value)
+        else:
+            raise GitConfigException('Value for "%s" is not a booleain: "%s"' %
+                                     (name, value))
 
     def getstartswith(self, name):
         self.load()
-        return ((n, v[-1]) for (n, v) in self.__cache.iteritems()
+        return ((n, v[-1]) for (n, v) in self.__cache.items()
                 if n.startswith(name))
 
     def rename_section(self, from_name, to_name):
@@ -124,8 +159,10 @@ class GitConfig:
         """Invoke 'git config --get-colorbool' and return the result."""
         return Run('git', 'config', '--get-colorbool', name,
                    stdout_is_tty).output_one_line()
-        
-config=GitConfig()
+
+
+config = GitConfig()
+
 
 def config_setup():
     global config
@@ -133,36 +170,3 @@ def config_setup():
     os.environ.setdefault('PAGER', config.get('stgit.pager'))
     os.environ.setdefault('LESS', '-FRSX')
     # FIXME: handle EDITOR the same way ?
-
-class ConfigOption:
-    """Delayed cached reading of a configuration option.
-    """
-    def __init__(self, section, option):
-        self.__section = section
-        self.__option = option
-        self.__value = None
-
-    def __str__(self):
-        if not self.__value:
-            self.__value = config.get(self.__section + '.' + self.__option)
-        return self.__value
-
-
-# cached extensions
-__extensions = None
-
-def file_extensions():
-    """Returns a dictionary with the conflict file extensions
-    """
-    global __extensions
-
-    if not __extensions:
-        cfg_ext = config.get('stgit.extensions').split()
-        if len(cfg_ext) != 3:
-            raise CmdException, '"extensions" configuration error'
-
-        __extensions = { 'ancestor': cfg_ext[0],
-                         'current':  cfg_ext[1],
-                         'patched':  cfg_ext[2] }
-
-    return __extensions
