@@ -1,5 +1,20 @@
-"""Python GIT interface
-"""
+# -*- coding: utf-8 -*-
+"""Python GIT interface"""
+
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import io
+import os
+import re
+import sys
+
+from stgit import basedir
+from stgit.compat import environ_get
+from stgit.config import config
+from stgit.exception import StgException
+from stgit.out import out
+from stgit.run import Run
+from stgit.utils import rename, strip_prefix
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -17,15 +32,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os, re
-from shutil import copyfile
-
-from stgit.exception import *
-from stgit import basedir
-from stgit.utils import *
-from stgit.out import *
-from stgit.run import *
-from stgit.config import config
 
 # git exception class
 class GitException(StgException):
@@ -46,7 +52,7 @@ class GRun(Run):
 # Classes
 #
 
-class Person:
+class Person(object):
     """An author, committer, etc."""
     def __init__(self, name = None, email = None, date = '',
                  desc = None):
@@ -76,9 +82,9 @@ class Person:
         if self.name and self.email:
             return '%s <%s>' % (self.name, self.email)
         else:
-            raise GitException, 'not enough identity data'
+            raise GitException('not enough identity data')
 
-class Commit:
+class Commit(object):
     """Handle the commit objects
     """
     def __init__(self, id_hash):
@@ -153,18 +159,11 @@ def get_conflicts():
     """Return the list of file conflicts
     """
     names = set()
-    for line in GRun('ls-files', '-z', '--unmerged'
-                     ).raw_output().split('\0')[:-1]:
+    for line in GRun('ls-files', '-z', '--unmerged').output_lines('\0'):
         stat, path = line.split('\t', 1)
         names.add(path)
     return list(names)
 
-def exclude_files():
-    files = [os.path.join(basedir.get(), 'info', 'exclude')]
-    user_exclude = config.get('core.excludesfile')
-    if user_exclude:
-        files.append(user_exclude)
-    return files
 
 def ls_files(files, tree = 'HEAD', full_name = True):
     """Return the files known to GIT or raise an error otherwise. It also
@@ -185,37 +184,29 @@ def ls_files(files, tree = 'HEAD', full_name = True):
         fileset = set(GRun('ls-files', '--error-unmatch', *args).output_lines())
     except GitRunException:
         # just hide the details of the 'git ls-files' command we use
-        raise GitException, \
-            'Some of the given paths are either missing or not known to GIT'
+        raise GitException(
+            'Some of the given paths are either missing or not known to GIT')
     return list(fileset)
 
-def parse_git_ls(output):
+def parse_git_ls(lines):
     """Parse the output of git diff-index, diff-files, etc. Doesn't handle
     rename/copy output, so don't feed it output generated with the -M
     or -C flags."""
     t = None
-    for line in output.split('\0'):
-        if not line:
-            # There's a zero byte at the end of the output, which
-            # gives us an empty string as the last "line".
-            continue
-        if t == None:
+    for line in lines:
+        if t is None:
             mode_a, mode_b, sha1_a, sha1_b, t = line.split(' ')
         else:
             yield (t, line)
             t = None
 
-def tree_status(files = None, tree_id = 'HEAD', unknown = False,
-                  noexclude = True, verbose = False):
+def tree_status(files=None, tree_id='HEAD', verbose=False):
     """Get the status of all changed files, or of a selected set of
     files. Returns a list of pairs - (status, filename).
 
-    If 'not files', it will check all files, and optionally all
-    unknown files.  If 'files' is a list, it will only check the files
-    in the list.
+    If 'not files', it will check all files. If 'files' is a list, it will only
+    check the files in the list.
     """
-    assert not files or not unknown
-
     if verbose:
         out.start('Checking for changes in the working directory')
 
@@ -225,21 +216,6 @@ def tree_status(files = None, tree_id = 'HEAD', unknown = False,
         files = []
     cache_files = []
 
-    # unknown files
-    if unknown:
-        cmd = ['ls-files', '-z', '--others', '--directory',
-               '--no-empty-directory']
-        if not noexclude:
-            cmd += ['--exclude=%s' % s for s in
-                    ['*.[ao]', '*.pyc', '.*', '*~', '#*', 'TAGS', 'tags']]
-            cmd += ['--exclude-per-directory=.gitignore']
-            cmd += ['--exclude-from=%s' % fn
-                    for fn in exclude_files()
-                    if os.path.exists(fn)]
-
-        lines = GRun(*cmd).raw_output().split('\0')
-        cache_files += [('?', line) for line in lines if line]
-
     # conflicted files
     conflicts = get_conflicts()
     cache_files += [('C', filename) for filename in conflicts
@@ -254,10 +230,11 @@ def tree_status(files = None, tree_id = 'HEAD', unknown = False,
         args = [tree_id]
         if files_left:
             args += ['--'] + files_left
-        for t, fn in parse_git_ls(GRun('diff-index', '-z', *args).raw_output()):
+        diff_index_lines = GRun('diff-index', '-z', *args).output_lines('\0')
+        for t, fn in parse_git_ls(diff_index_lines):
             # the condition is needed in case files is emtpy and
             # diff-index lists those already reported
-            if not fn in reported_files:
+            if fn not in reported_files:
                 cache_files.append((t, fn))
                 reported_files.add(fn)
         files_left = [f for f in files if f not in reported_files]
@@ -270,10 +247,11 @@ def tree_status(files = None, tree_id = 'HEAD', unknown = False,
         args = []
         if files_left:
             args += ['--'] + files_left
-        for t, fn in parse_git_ls(GRun('diff-files', '-z', *args).raw_output()):
+        diff_files_lines = GRun('diff-files', '-z', *args).output_lines('\0')
+        for t, fn in parse_git_ls(diff_files_lines):
             # the condition is needed in case files is empty and
             # diff-files lists those already reported
-            if not fn in reported_files:
+            if fn not in reported_files:
                 cache_files.append((t, fn))
                 reported_files.add(fn)
 
@@ -330,14 +308,14 @@ def set_head_file(ref):
     try:
         GRun('symbolic-ref', 'HEAD', 'refs/heads/%s' % ref).run()
     except GitRunException:
-        raise GitException, 'Could not set head to "%s"' % ref
+        raise GitException('Could not set head to "%s"' % ref)
 
 def set_ref(ref, val):
     """Point ref at a new commit object."""
     try:
         GRun('update-ref', ref, val).run()
     except GitRunException:
-        raise GitException, 'Could not update %s to "%s".' % (ref, val)
+        raise GitException('Could not update %s to "%s".' % (ref, val))
 
 def set_branch(branch, val):
     set_ref('refs/heads/%s' % branch, val)
@@ -373,7 +351,7 @@ def rev_parse(git_id):
         return GRun('rev-parse', '--verify', git_id
                     ).discard_stderr().output_one_line()
     except GitRunException:
-        raise GitException, 'Unknown revision: %s' % git_id
+        raise GitException('Unknown revision: %s' % git_id)
 
 def ref_exists(ref):
     try:
@@ -389,7 +367,7 @@ def create_branch(new_branch, tree_id = None):
     """Create a new branch in the git repository
     """
     if branch_exists(new_branch):
-        raise GitException, 'Branch "%s" already exists' % new_branch
+        raise GitException('Branch "%s" already exists' % new_branch)
 
     current_head_file = get_head_file()
     current_head = get_head()
@@ -415,7 +393,7 @@ def switch_branch(new_branch):
     global __head
 
     if not branch_exists(new_branch):
-        raise GitException, 'Branch "%s" does not exist' % new_branch
+        raise GitException('Branch "%s" does not exist' % new_branch)
 
     tree_id = rev_parse('refs/heads/%s^{commit}' % new_branch)
     if tree_id != get_head():
@@ -423,7 +401,7 @@ def switch_branch(new_branch):
         try:
             GRun('read-tree', '-u', '-m', get_head(), tree_id).run()
         except GitRunException:
-            raise GitException, 'read-tree failed (local changes maybe?)'
+            raise GitException('read-tree failed (local changes maybe?)')
         __head = tree_id
     set_head_file(new_branch)
 
@@ -432,31 +410,31 @@ def switch_branch(new_branch):
 
 def delete_ref(ref):
     if not ref_exists(ref):
-        raise GitException, '%s does not exist' % ref
+        raise GitException('%s does not exist' % ref)
     sha1 = GRun('show-ref', '-s', ref).output_one_line()
     try:
         GRun('update-ref', '-d', ref, sha1).run()
     except GitRunException:
-        raise GitException, 'Failed to delete ref %s' % ref
+        raise GitException('Failed to delete ref %s' % ref)
 
 def delete_branch(name):
     delete_ref('refs/heads/%s' % name)
 
 def rename_ref(from_ref, to_ref):
     if not ref_exists(from_ref):
-        raise GitException, '"%s" does not exist' % from_ref
+        raise GitException('"%s" does not exist' % from_ref)
     if ref_exists(to_ref):
-        raise GitException, '"%s" already exists' % to_ref
+        raise GitException('"%s" already exists' % to_ref)
 
     sha1 = GRun('show-ref', '-s', from_ref).output_one_line()
     try:
         GRun('update-ref', to_ref, sha1, '0'*40).run()
     except GitRunException:
-        raise GitException, 'Failed to create new ref %s' % to_ref
+        raise GitException('Failed to create new ref %s' % to_ref)
     try:
         GRun('update-ref', '-d', from_ref, sha1).run()
     except GitRunException:
-        raise GitException, 'Failed to delete ref %s' % from_ref
+        raise GitException('Failed to delete ref %s' % from_ref)
 
 def rename_branch(from_name, to_name):
     """Rename a git branch."""
@@ -484,24 +462,21 @@ def user():
         name=config.get('user.name')
         email=config.get('user.email')
         __user = Person(name, email)
-    return __user;
+    return __user
 
 def author():
     """Return the author information.
     """
     global __author
     if not __author:
-        try:
-            # the environment variables take priority over config
-            try:
-                date = os.environ['GIT_AUTHOR_DATE']
-            except KeyError:
-                date = ''
-            __author = Person(os.environ['GIT_AUTHOR_NAME'],
-                              os.environ['GIT_AUTHOR_EMAIL'],
-                              date)
-        except KeyError:
+        # the environment variables take priority over config
+        name = environ_get('GIT_AUTHOR_NAME')
+        email = environ_get('GIT_AUTHOR_EMAIL')
+        if name is None or email is None:
             __author = user()
+        else:
+            date = environ_get('GIT_AUTHOR_DATE', '')
+            __author = Person(name, email, date)
     return __author
 
 def committer():
@@ -509,17 +484,14 @@ def committer():
     """
     global __committer
     if not __committer:
-        try:
-            # the environment variables take priority over config
-            try:
-                date = os.environ['GIT_COMMITTER_DATE']
-            except KeyError:
-                date = ''
-            __committer = Person(os.environ['GIT_COMMITTER_NAME'],
-                                 os.environ['GIT_COMMITTER_EMAIL'],
-                                 date)
-        except KeyError:
+        # the environment variables take priority over config
+        name = environ_get('GIT_COMMITTER_NAME')
+        email = environ_get('GIT_COMMITTER_EMAIL')
+        if name is None or email is None:
             __committer = user()
+        else:
+            date = environ_get('GIT_COMMITTER_DATE', '')
+            __committer = Person(name, email, date)
     return __committer
 
 def update_cache(files = None, force = False):
@@ -534,7 +506,7 @@ def update_cache(files = None, force = False):
     # check for unresolved conflicts
     if not force and [x for x in cache_files
                       if x[0] not in ['M', 'N', 'A', 'D']]:
-        raise GitException, 'Updating cache failed: unresolved conflicts'
+        raise GitException('Updating cache failed: unresolved conflicts')
 
     # update the cache
     add_files = [x[1] for x in cache_files if x[0] in ['N', 'A']]
@@ -560,7 +532,7 @@ def commit(message, files = None, parents = None, allowempty = False,
     if cache_update and parents != []:
         changes = update_cache(files)
         if not changes and not allowempty:
-            raise GitException, 'No changes to commit'
+            raise GitException('No changes to commit')
 
     # get the commit message
     if not message:
@@ -569,7 +541,7 @@ def commit(message, files = None, parents = None, allowempty = False,
         message += '\n'
 
     # write the index to repository
-    if tree_id == None:
+    if tree_id is None:
         tree_id = GRun('write-tree').output_one_line()
         set_head = True
 
@@ -610,7 +582,7 @@ def apply_diff(rev1, rev2, check_index = True, files = None):
     diff_str = diff(files, rev1, rev2)
     if diff_str:
         try:
-            GRun('apply', *index_opt).raw_input(
+            GRun('apply', *index_opt).encoding(None).raw_input(
                 diff_str).discard_stderr().no_output()
         except GitRunException:
             return False
@@ -631,12 +603,12 @@ def merge_recursive(base, head1, head2):
     output = p.output_lines()
     if p.exitcode:
         # There were conflicts
-        if config.get('stgit.autoimerge') == 'yes':
+        if config.getbool('stgit.autoimerge'):
             mergetool()
         else:
             conflicts = [l for l in output if l.startswith('CONFLICT')]
             out.info(*conflicts)
-            raise GitException, "%d conflict(s)" % len(conflicts)
+            raise GitException("%d conflict(s)" % len(conflicts))
 
 def mergetool(files = ()):
     """Invoke 'git mergetool' to resolve any outstanding conflicts. If 'not
@@ -647,7 +619,7 @@ def mergetool(files = ()):
     conflicts = ['CONFLICT ' + f for f in get_conflicts()]
     if conflicts:
         out.info(*conflicts)
-        raise GitException, "%d conflict(s)" % len(conflicts)
+        raise GitException("%d conflict(s)" % len(conflicts))
 
 def diff(files = None, rev1 = 'HEAD', rev2 = None, diff_flags = [],
          binary = True):
@@ -659,18 +631,20 @@ def diff(files = None, rev1 = 'HEAD', rev2 = None, diff_flags = [],
         diff_flags = diff_flags + ['--binary']
 
     if rev1 and rev2:
-        return GRun('diff-tree', '-p',
-                    *(diff_flags + [rev1, rev2, '--'] + files)).raw_output()
+        runner = GRun('diff-tree', '-p',
+                      *(diff_flags + [rev1, rev2, '--'] + files))
+        return runner.decoding(None).raw_output()
     elif rev1 or rev2:
         refresh_index()
         if rev2:
-            return GRun('diff-index', '-p', '-R',
-                        *(diff_flags + [rev2, '--'] + files)).raw_output()
+            runner = GRun('diff-index', '-p', '-R',
+                          *(diff_flags + [rev2, '--'] + files))
         else:
-            return GRun('diff-index', '-p',
-                        *(diff_flags + [rev1, '--'] + files)).raw_output()
+            runner = GRun('diff-index', '-p',
+                          *(diff_flags + [rev1, '--'] + files))
+        return runner.decoding(None).raw_output()
     else:
-        return ''
+        return b''
 
 def files(rev1, rev2, diff_flags = []):
     """Return the files modified between rev1 and rev2
@@ -693,27 +667,13 @@ def barefiles(rev1, rev2):
 
     return '\n'.join(result)
 
-def pretty_commit(commit_id = 'HEAD', flags = []):
-    """Return a given commit (log + diff)
+def checkout(tree_id):
+    """Check out the given tree_id
     """
-    return GRun('show', *(flags + [commit_id])).raw_output()
-
-def checkout(files = None, tree_id = None, force = False):
-    """Check out the given or all files
-    """
-    if tree_id:
-        try:
-            GRun('read-tree', '--reset', tree_id).run()
-        except GitRunException:
-            raise GitException, 'Failed "git read-tree" --reset %s' % tree_id
-
-    cmd = ['checkout-index', '-q', '-u']
-    if force:
-        cmd.append('-f')
-    if files:
-        GRun(*(cmd + ['--'])).xargs(files)
-    else:
-        GRun(*(cmd + ['-a'])).run()
+    try:
+        GRun('read-tree', '--reset', '-u', tree_id).run()
+    except GitRunException:
+        raise GitException('Failed "git read-tree" --reset %s' % tree_id)
 
 def switch(tree_id, keep = False):
     """Switch the tree to the given id
@@ -726,33 +686,20 @@ def switch(tree_id, keep = False):
         try:
             GRun('read-tree', '-u', '-m', get_head(), tree_id).run()
         except GitRunException:
-            raise GitException, 'read-tree failed (local changes maybe?)'
+            raise GitException('read-tree failed (local changes maybe?)')
 
     __set_head(tree_id)
 
-def reset(files = None, tree_id = None, check_out = True):
+def reset(tree_id = None):
     """Revert the tree changes relative to the given tree_id. It removes
     any local changes
     """
     if not tree_id:
         tree_id = get_head()
 
-    if check_out:
-        checkout(files, tree_id, True)
+    checkout(tree_id)
+    __set_head(tree_id)
 
-    # if the reset refers to the whole tree, switch the HEAD as well
-    if not files:
-        __set_head(tree_id)
-
-def resolved(filenames, reset = None):
-    if reset:
-        stage = {'ancestor': 1, 'current': 2, 'patched': 3}[reset]
-        GRun('checkout-index', '--no-create', '--stage=%d' % stage,
-             '--stdin', '-z').input_nulterm(filenames).no_output()
-    GRun('update-index', '--add', '--').xargs(filenames)
-    for filename in filenames:
-        # update the access and modificatied times
-        os.utime(filename, None)
 
 def fetch(repository = 'origin', refspec = None):
     """Fetches changes from the remote repository, using 'git fetch'
@@ -796,14 +743,14 @@ def rebase(tree_id = None):
     elif command:
         args = []
     else:
-        raise GitException, 'Default rebasing requires a commit id'
+        raise GitException('Default rebasing requires a commit id')
     if command:
         # clear the HEAD cache as the custom rebase command will update it
         __clear_head_cache()
         Run(*(command.split() + args)).run()
     else:
         # default rebasing
-        reset(tree_id = tree_id)
+        reset(tree_id)
 
 def repack():
     """Repack all objects into a single pack
@@ -817,12 +764,13 @@ def apply_patch(filename = None, diff = None, base = None,
     """
     if diff is None:
         if filename:
-            f = file(filename)
+            with io.open(filename, 'rb') as f:
+                diff = f.read()
         else:
-            f = sys.stdin
-        diff = f.read()
-        if filename:
-            f.close()
+            if hasattr(sys.stdin, 'buffer'):
+                diff = sys.stdin.buffer.read()
+            else:
+                diff = sys.stdin.read()
 
     if base:
         orig_head = get_head()
@@ -833,10 +781,10 @@ def apply_patch(filename = None, diff = None, base = None,
     cmd = ['apply', '--index']
     if reject:
         cmd += ['--reject']
-    if strip != None:
+    if strip is not None:
         cmd += ['-p%s' % (strip,)]
     try:
-        GRun(*cmd).raw_input(diff).no_output()
+        GRun(*cmd).encoding(None).raw_input(diff).no_output()
     except GitRunException:
         if base:
             switch(orig_head)
@@ -848,11 +796,6 @@ def apply_patch(filename = None, diff = None, base = None,
         switch(orig_head)
         merge_recursive(base, orig_head, top)
 
-def clone(repository, local_dir):
-    """Clone a remote repository. At the moment, just use the
-    'git clone' script
-    """
-    GRun('clone', repository, local_dir).run()
 
 def modifying_revs(files, base_rev, head_rev):
     """Return the revisions from the list modifying the given files."""
@@ -866,13 +809,6 @@ def refspec_localpart(refspec):
     else:
         raise GitException('Cannot parse refspec "%s"' % refspec)
 
-def refspec_remotepart(refspec):
-    m = re.match('^([^:]*):[^:]*$', refspec)
-    if m:
-        return m.group(1)
-    else:
-        raise GitException('Cannot parse refspec "%s"' % refspec)
-
 def __remotes_from_config():
     return config.sections_matching(r'remote\.(.*)\.url')
 
@@ -908,9 +844,9 @@ def remotes_local_branches(remote):
         stream.close()
     elif remote in __remotes_from_dir('branches'):
         # old-style branches only declare one branch
-        branches.append('refs/heads/'+remote);
+        branches.append('refs/heads/'+remote)
     else:
-        raise GitException, 'Unknown remote "%s"' % remote
+        raise GitException('Unknown remote "%s"' % remote)
 
     return branches
 
@@ -938,7 +874,7 @@ def fetch_head():
         m = re.match('^([^\t]*)\t\t', line)
         if m:
             if fetch_head:
-                raise GitException, 'StGit does not support multiple FETCH_HEAD'
+                raise GitException('StGit does not support multiple FETCH_HEAD')
             else:
                 fetch_head=m.group(1)
     stream.close()
