@@ -1,12 +1,21 @@
+# -*- coding: utf-8 -*-
 """A Python class hierarchy wrapping a git repository and its
 contents."""
 
-import atexit, os, os.path, re, signal
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 from datetime import datetime, timedelta, tzinfo
+import atexit
+import os
+import re
+import signal
 
-from stgit import exception, run, utils
+from stgit import exception, utils
+from stgit.run import Run, RunException
+from stgit.compat import environ_get, text
 from stgit.config import config
 
+
 class Immutable(object):
     """I{Immutable} objects cannot be modified once created. Any
     modification methods will return a new object, leaving the
@@ -94,16 +103,16 @@ def system_date(datestring):
         # would convert it to the local time zone.
         (ds, z) = m.groups()
         try:
-            t = run.Run("date", "+%Y-%m-%d-%H-%M-%S", "-d", ds
-                        ).output_one_line()
-        except run.RunException:
+            t = Run("date", "+%Y-%m-%d-%H-%M-%S", "-d", ds
+                    ).output_one_line()
+        except RunException:
             return None
     else:
         # Time zone not included; we ask "date" to provide it for us.
         try:
-            d = run.Run("date", "+%Y-%m-%d-%H-%M-%S_%z", "-d", datestring
-                        ).output_one_line()
-        except run.RunException:
+            d = Run("date", "+%Y-%m-%d-%H-%M-%S_%z", "-d", datestring
+                    ).output_one_line()
+        except RunException:
             return None
         (t, z) = d.split("_")
     year, month, day, hour, minute, second = [int(x) for x in t.split("-")]
@@ -132,7 +141,7 @@ class Date(Immutable, Repr):
         if m:
             try:
                 self.__time = datetime(
-                    *[int(m.group(i + 1)) for i in xrange(6)],
+                    *[int(m.group(i + 1)) for i in range(6)],
                     **{'tzinfo': TimeZone(m.group(7))})
             except ValueError:
                 raise DateException(datestring, 'date')
@@ -149,7 +158,7 @@ class Date(Immutable, Repr):
         return self.isoformat()
     def isoformat(self):
         """Human-friendly ISO 8601 format."""
-        return '%s %s' % (self.__time.replace(tzinfo = None).isoformat(' '),
+        return '%s %s' % (self.__time.replace(tzinfo=None).isoformat(str(' ')),
                           self.__time.tzinfo)
     @classmethod
     def maybe(cls, datestring):
@@ -169,10 +178,23 @@ class Person(Immutable, Repr):
         self.__email = d(email, 'email')
         self.__date = d(date, 'date')
         assert isinstance(self.__date, Date) or self.__date in [None, NoValue]
-    name = property(lambda self: self.__name)
-    email = property(lambda self: self.__email)
-    name_email = property(lambda self: '%s <%s>' % (self.name, self.email))
-    date = property(lambda self: self.__date)
+
+    @property
+    def name(self):
+        return self.__name
+
+    @property
+    def email(self):
+        return self.__email
+
+    @property
+    def name_email(self):
+        return '%s <%s>' % (self.name, self.email)
+
+    @property
+    def date(self):
+        return self.__date
+
     def set_name(self, name):
         return type(self)(name = name, defaults = self)
     def set_email(self, email):
@@ -199,20 +221,19 @@ class Person(Immutable, Repr):
     def author(cls):
         if not hasattr(cls, '__author'):
             cls.__author = cls(
-                name = os.environ.get('GIT_AUTHOR_NAME', NoValue),
-                email = os.environ.get('GIT_AUTHOR_EMAIL', NoValue),
-                date = Date.maybe(os.environ.get('GIT_AUTHOR_DATE', NoValue)),
-                defaults = cls.user())
+                name=environ_get('GIT_AUTHOR_NAME', NoValue),
+                email=environ_get('GIT_AUTHOR_EMAIL', NoValue),
+                date=Date.maybe(environ_get('GIT_AUTHOR_DATE', NoValue)),
+                defaults=cls.user())
         return cls.__author
     @classmethod
     def committer(cls):
         if not hasattr(cls, '__committer'):
             cls.__committer = cls(
-                name = os.environ.get('GIT_COMMITTER_NAME', NoValue),
-                email = os.environ.get('GIT_COMMITTER_EMAIL', NoValue),
-                date = Date.maybe(
-                    os.environ.get('GIT_COMMITTER_DATE', NoValue)),
-                defaults = cls.user())
+                name=environ_get('GIT_COMMITTER_NAME', NoValue),
+                email=environ_get('GIT_COMMITTER_EMAIL', NoValue),
+                date=Date.maybe(environ_get('GIT_COMMITTER_DATE', NoValue)),
+                defaults=cls.user())
         return cls.__committer
 
 class GitObject(Immutable, Repr):
@@ -223,15 +244,20 @@ class GitObject(Immutable, Repr):
 
 class BlobData(Immutable, Repr):
     """Represents the data contents of a git blob object."""
-    def __init__(self, string):
-        self.__string = str(string)
-    str = property(lambda self: self.__string)
+    def __init__(self, data):
+        assert isinstance(data, bytes)
+        self.__bytes = data
+
+    @property
+    def bytes(self):
+        return self.__bytes
+
     def commit(self, repository):
         """Commit the blob.
         @return: The committed blob
         @rtype: L{Blob}"""
-        sha1 = repository.run(['git', 'hash-object', '-w', '--stdin']
-                              ).raw_input(self.str).output_one_line()
+        runner = repository.run(['git', 'hash-object', '-w', '--stdin'])
+        sha1 = runner.encoding(None).raw_input(self.bytes).output_one_line()
         return repository.get_blob(sha1)
 
 class Blob(GitObject):
@@ -243,12 +269,16 @@ class Blob(GitObject):
     def __init__(self, repository, sha1):
         self.__repository = repository
         self.__sha1 = sha1
-    sha1 = property(lambda self: self.__sha1)
+
+    @property
+    def sha1(self):
+        return self.__sha1
+
     def __str__(self):
         return 'Blob<%s>' % self.sha1
     @property
     def data(self):
-        return BlobData(self.__repository.cat_object(self.sha1))
+        return BlobData(self.__repository.cat_object(self.sha1, encoding=None))
 
 class ImmutableDict(dict):
     """A dictionary that cannot be modified once it's been created."""
@@ -276,51 +306,29 @@ class TreeData(Immutable, Repr):
         (strings) to either (I{permission}, I{object}) tuples or just
         objects."""
         self.__entries = ImmutableDict((name, self.__x(po))
-                                       for (name, po) in entries.iteritems())
-    entries = property(lambda self: self.__entries)
-    """Map from name to (I{permission}, I{object}) tuple."""
-    def set_entry(self, name, po):
-        """Create a new L{TreeData} object identical to this one, except that
-        it maps C{name} to C{po}.
+                                       for (name, po) in entries.items())
 
-        @param name: Name of the changed mapping
-        @type name: C{str}
-        @param po: Value of the changed mapping
-        @type po: L{Blob} or L{Tree} or (C{str}, L{Blob} or L{Tree})
-        @return: The new L{TreeData} object
-        @rtype: L{TreeData}"""
-        e = dict(self.entries)
-        e[name] = self.__x(po)
-        return type(self)(e)
-    def del_entry(self, name):
-        """Create a new L{TreeData} object identical to this one, except that
-        it doesn't map C{name} to anything.
+    @property
+    def entries(self):
+        return self.__entries
 
-        @param name: Name of the deleted mapping
-        @type name: C{str}
-        @return: The new L{TreeData} object
-        @rtype: L{TreeData}"""
-        e = dict(self.entries)
-        del e[name]
-        return type(self)(e)
     def commit(self, repository):
         """Commit the tree.
         @return: The committed tree
         @rtype: L{Tree}"""
-        listing = ''.join(
-            '%s %s %s\t%s\0' % (mode, obj.typename, obj.sha1, name)
-            for (name, (mode, obj)) in self.entries.iteritems())
+        listing = ['%s %s %s\t%s' % (mode, obj.typename, obj.sha1, name)
+                   for (name, (mode, obj)) in self.entries.items()]
         sha1 = repository.run(['git', 'mktree', '-z']
-                              ).raw_input(listing).output_one_line()
+                              ).input_nulterm(listing).output_one_line()
         return repository.get_tree(sha1)
     @classmethod
-    def parse(cls, repository, s):
+    def parse(cls, repository, lines):
         """Parse a raw git tree description.
 
         @return: A new L{TreeData} object
         @rtype: L{TreeData}"""
         entries = {}
-        for line in s.split('\0')[:-1]:
+        for line in lines:
             m = re.match(r'^([0-7]{6}) ([a-z]+) ([0-9a-f]{40})\t(.*)$', line)
             assert m
             perm, type, sha1, name = m.groups()
@@ -337,14 +345,18 @@ class Tree(GitObject):
         self.__sha1 = sha1
         self.__repository = repository
         self.__data = None
-    sha1 = property(lambda self: self.__sha1)
+
+    @property
+    def sha1(self):
+        return self.__sha1
+
     @property
     def data(self):
-        if self.__data == None:
+        if self.__data is None:
             self.__data = TreeData.parse(
                 self.__repository,
                 self.__repository.run(['git', 'ls-tree', '-z', self.sha1]
-                                      ).raw_output())
+                                      ).output_lines('\0'))
         return self.__data
     def __str__(self):
         return 'Tree<sha1: %s>' % self.sha1
@@ -364,21 +376,38 @@ class CommitData(Immutable, Repr):
         env = {}
         for p, v1 in ((self.author, 'AUTHOR'),
                        (self.committer, 'COMMITTER')):
-            if p != None:
+            if p is not None:
                 for attr, v2 in (('name', 'NAME'), ('email', 'EMAIL'),
                                  ('date', 'DATE')):
-                    if getattr(p, attr) != None:
-                        env['GIT_%s_%s' % (v1, v2)] = str(getattr(p, attr))
+                    if getattr(p, attr) is not None:
+                        env['GIT_%s_%s' % (v1, v2)] = text(getattr(p, attr))
         return env
-    tree = property(lambda self: self.__tree)
-    parents = property(lambda self: self.__parents)
+
+    @property
+    def tree(self):
+        return self.__tree
+
+    @property
+    def parents(self):
+        return self.__parents
+
     @property
     def parent(self):
         assert len(self.__parents) == 1
         return self.__parents[0]
-    author = property(lambda self: self.__author)
-    committer = property(lambda self: self.__committer)
-    message = property(lambda self: self.__message)
+
+    @property
+    def author(self):
+        return self.__author
+
+    @property
+    def committer(self):
+        return self.__committer
+
+    @property
+    def message(self):
+        return self.__message
+
     def set_tree(self, tree):
         return type(self)(tree = tree, defaults = self)
     def set_parents(self, parents):
@@ -397,11 +426,11 @@ class CommitData(Immutable, Repr):
     def is_nochange(self):
         return len(self.parents) == 1 and self.tree == self.parent.data.tree
     def __str__(self):
-        if self.tree == None:
+        if self.tree is None:
             tree = None
         else:
             tree = self.tree.sha1
-        if self.parents == None:
+        if self.parents is None:
             parents = None
         else:
             parents = [p.sha1 for p in self.parents]
@@ -460,10 +489,14 @@ class Commit(GitObject):
         self.__sha1 = sha1
         self.__repository = repository
         self.__data = None
-    sha1 = property(lambda self: self.__sha1)
+
+    @property
+    def sha1(self):
+        return self.__sha1
+
     @property
     def data(self):
-        if self.__data == None:
+        if self.__data is None:
             self.__data = CommitData.parse(
                 self.__repository,
                 self.__repository.cat_object(self.sha1))
@@ -483,7 +516,7 @@ class Refs(object):
         runner = self.__repository.run(['git', 'show-ref'])
         try:
             lines = runner.output_lines()
-        except run.RunException:
+        except RunException:
             # as this happens both in non-git trees and empty git
             # trees, we silently ignore this error
             return
@@ -494,7 +527,7 @@ class Refs(object):
     def get(self, ref):
         """Get the Commit the given ref points to. Throws KeyError if ref
         doesn't exist."""
-        if self.__refs == None:
+        if self.__refs is None:
             self.__cache_refs()
         return self.__repository.get_commit(self.__refs[ref])
     def exists(self, ref):
@@ -508,7 +541,7 @@ class Refs(object):
     def set(self, ref, commit, msg):
         """Write the sha1 of the given Commit to the ref. The ref may or may
         not already exist."""
-        if self.__refs == None:
+        if self.__refs is None:
             self.__cache_refs()
         old_sha1 = self.__refs.get(ref, '0'*40)
         new_sha1 = commit.sha1
@@ -518,7 +551,7 @@ class Refs(object):
             self.__refs[ref] = new_sha1
     def delete(self, ref):
         """Delete the given ref. Throws KeyError if ref doesn't exist."""
-        if self.__refs == None:
+        if self.__refs is None:
             self.__cache_refs()
         self.__repository.run(['git', 'update-ref',
                                '-d', ref, self.__refs[ref]]).no_output()
@@ -532,13 +565,13 @@ class ObjectCache(object):
         self.__objects = {}
         self.__create = create
     def __getitem__(self, name):
-        if not name in self.__objects:
+        if name not in self.__objects:
             self.__objects[name] = self.__create(name)
         return self.__objects[name]
     def __contains__(self, name):
         return name in self.__objects
     def __setitem__(self, name, val):
-        assert not name in self.__objects
+        assert name not in self.__objects
         self.__objects[name] = val
 
 class RunWithEnv(object):
@@ -549,7 +582,7 @@ class RunWithEnv(object):
         @param args: Command and argument vector
         @type env: dict
         @param env: Extra environment"""
-        return run.Run(*args).env(utils.add_dict(self.env, env))
+        return Run(*args).env(utils.add_dict(self.env, env))
 
 class RunWithEnvCwd(RunWithEnv):
     def run(self, args, env = {}):
@@ -586,27 +619,32 @@ class CatFileProcess(object):
             p.stdin.close()
             os.kill(p.pid(), signal.SIGTERM)
             p.wait()
-    def cat_file(self, sha1):
+    def cat_file(self, sha1, encoding):
         p = self.__get_process()
         p.stdin.write('%s\n' % sha1)
         p.stdin.flush()
 
         # Read until we have the entire status line.
-        s = ''
-        while not '\n' in s:
+        s = b''
+        while b'\n' not in s:
             s += os.read(p.stdout.fileno(), 4096)
-        h, b = s.split('\n', 1)
-        if h == '%s missing' % sha1:
-            raise SomeException()
-        hash, type, length = h.split()
-        assert hash == sha1
-        length = int(length)
+        h, b = s.split(b'\n', 1)
+        header = h.decode('utf-8')
+        if header == '%s missing' % sha1:
+            raise RepositoryException('Cannot cat %s' % sha1)
+        name, type_, size = header.split()
+        assert name == sha1
+        size = int(size)
 
         # Read until we have the entire object plus the trailing
         # newline.
-        while len(b) < length + 1:
+        while len(b) < size + 1:
             b += os.read(p.stdout.fileno(), 4096)
-        return type, b[:-1]
+        content = b[:size]
+        if encoding:
+            return type_, content.decode(encoding)
+        else:
+            return type_, content
 
 class DiffTreeProcesses(object):
     def __init__(self, repo):
@@ -615,9 +653,10 @@ class DiffTreeProcesses(object):
         atexit.register(self.__shutdown)
     def __get_process(self, args):
         args = tuple(args)
-        if not args in self.__procs:
+        if args not in self.__procs:
             self.__procs[args] = self.__repo.run(
-                ['git', 'diff-tree', '--stdin'] + list(args)).run_background()
+                ['git', 'diff-tree', '--stdin', '--encoding=utf-8'] +
+                list(args)).run_background()
         return self.__procs[args]
     def __shutdown(self):
         for p in self.__procs.values():
@@ -626,15 +665,18 @@ class DiffTreeProcesses(object):
     def diff_trees(self, args, sha1a, sha1b):
         p = self.__get_process(args)
         query = '%s %s\n' % (sha1a, sha1b)
-        end = 'EOF\n' # arbitrary string that's not a 40-digit hex number
-        p.stdin.write(query + end)
+        b_query = query.encode('utf-8')
+        end = 'EOF\n'  # arbitrary string that's not a 40-digit hex number
+        b_end = end.encode('utf-8')
+        os.write(p.stdin.fileno(), b_query + b_end)
         p.stdin.flush()
-        s = ''
-        while not (s.endswith('\n' + end) or s.endswith('\0' + end)):
-            s += os.read(p.stdout.fileno(), 4096)
-        assert s.startswith(query)
-        assert s.endswith(end)
-        return s[len(query):-len(end)]
+        data = bytes()
+        while not (data.endswith(b'\n' + b_end) or
+                   data.endswith(b'\0' + b_end)):
+            data += os.read(p.stdout.fileno(), 4096)
+        assert data.startswith(b_query)
+        assert data.endswith(b_end)
+        return data[len(b_query):-len(b_end)]
 
 class Repository(RunWithEnv):
     """Represents a git repository."""
@@ -649,14 +691,17 @@ class Repository(RunWithEnv):
         self.__default_iw = None
         self.__catfile = CatFileProcess(self)
         self.__difftree = DiffTreeProcesses(self)
-    env = property(lambda self: { 'GIT_DIR': self.__git_dir })
+
+    @property
+    def env(self):
+        return {'GIT_DIR': self.__git_dir}
+
     @classmethod
     def default(cls):
         """Return the default repository."""
         try:
-            return cls(run.Run('git', 'rev-parse', '--git-dir'
-                               ).output_one_line())
-        except run.RunException:
+            return cls(Run('git', 'rev-parse', '--git-dir').output_one_line())
+        except RunException:
             raise RepositoryException('Cannot find git repository')
     @property
     def current_branch_name(self):
@@ -666,9 +711,9 @@ class Repository(RunWithEnv):
     def default_index(self):
         """An L{Index} object representing the default index file for the
         repository."""
-        if self.__default_index == None:
+        if self.__default_index is None:
             self.__default_index = Index(
-                self, (os.environ.get('GIT_INDEX_FILE', None)
+                self, (environ_get('GIT_INDEX_FILE', None)
                        or os.path.join(self.__git_dir, 'index')))
         return self.__default_index
     def temp_index(self):
@@ -678,10 +723,10 @@ class Repository(RunWithEnv):
     @property
     def default_worktree(self):
         """A L{Worktree} object representing the default work tree."""
-        if self.__default_worktree == None:
-            path = os.environ.get('GIT_WORK_TREE', None)
+        if self.__default_worktree is None:
+            path = environ_get('GIT_WORK_TREE', None)
             if not path:
-                o = run.Run('git', 'rev-parse', '--show-cdup').output_lines()
+                o = Run('git', 'rev-parse', '--show-cdup').output_lines()
                 o = o or ['.']
                 assert len(o) == 1
                 path = o[0]
@@ -691,14 +736,22 @@ class Repository(RunWithEnv):
     def default_iw(self):
         """An L{IndexAndWorktree} object representing the default index and
         work tree for this repository."""
-        if self.__default_iw == None:
+        if self.__default_iw is None:
             self.__default_iw = IndexAndWorktree(self.default_index,
                                                  self.default_worktree)
         return self.__default_iw
-    directory = property(lambda self: self.__git_dir)
-    refs = property(lambda self: self.__refs)
-    def cat_object(self, sha1):
-        return self.__catfile.cat_file(sha1)[1]
+
+    @property
+    def directory(self):
+        return self.__git_dir
+
+    @property
+    def refs(self):
+        return self.__refs
+
+    def cat_object(self, sha1, encoding='utf-8'):
+        return self.__catfile.cat_file(sha1, encoding)[1]
+
     def rev_parse(self, rev, discard_stderr = False, object_type = 'commit'):
         assert object_type in ('commit', 'tree', 'blob')
         getter = getattr(self, 'get_' + object_type)
@@ -706,7 +759,7 @@ class Repository(RunWithEnv):
             return getter(self.run(
                     ['git', 'rev-parse', '%s^{%s}' % (rev, object_type)]
                     ).discard_stderr(discard_stderr).output_one_line())
-        except run.RunException:
+        except RunException:
             raise RepositoryException('%s: No such %s' % (rev, object_type))
     def get_blob(self, sha1):
         return self.__blobs[sha1]
@@ -725,7 +778,7 @@ class Repository(RunWithEnv):
         try:
             return self.run(['git', 'symbolic-ref', '-q', 'HEAD']
                             ).output_one_line()
-        except run.RunException:
+        except RunException:
             raise DetachedHeadException()
     def set_head_ref(self, ref, msg):
         self.run(['git', 'symbolic-ref', '-m', msg, 'HEAD', ref]).no_output()
@@ -745,23 +798,32 @@ class Repository(RunWithEnv):
         finally:
             index.delete()
         return result
-    def apply(self, tree, patch_text, quiet):
+    def apply(self, tree, patch_bytes, quiet):
         """Given a L{Tree} and a patch, will either return the new L{Tree}
         that results when the patch is applied, or None if the patch
         couldn't be applied."""
         assert isinstance(tree, Tree)
-        if not patch_text:
+        if not patch_bytes:
             return tree
         index = self.temp_index()
         try:
             index.read_tree(tree)
             try:
-                index.apply(patch_text, quiet)
+                index.apply(patch_bytes, quiet)
                 return index.write_tree()
             except MergeException:
                 return None
         finally:
             index.delete()
+    def submodules(self, tree):
+        """Given a L{Tree}, return list of paths which are submodules."""
+        assert isinstance(tree, Tree)
+        # A simple regex to match submodule entries
+        regex = re.compile(r'160000 commit [0-9a-f]{40}\t(.*)$')
+        # First, use ls-tree to get all the trees and links
+        files = self.run(['git', 'ls-tree', '-d', '-r', '-z', tree.sha1]).output_lines('\0')
+        # Then extract the paths of any submodules
+        return set(m.group(1) for m in map(regex.match, files) if m)
     def diff_tree(self, t1, t2, diff_opts, binary = True):
         """Given two L{Tree}s C{t1} and C{t2}, return the patch that takes
         C{t1} to C{t2}.
@@ -773,7 +835,7 @@ class Repository(RunWithEnv):
         assert isinstance(t1, Tree)
         assert isinstance(t2, Tree)
         diff_opts = list(diff_opts)
-        if binary and not '--binary' in diff_opts:
+        if binary and '--binary' not in diff_opts:
             diff_opts.append('--binary')
         return self.__difftree.diff_trees(['-p'] + diff_opts,
                                           t1.sha1, t2.sha1)
@@ -786,17 +848,17 @@ class Repository(RunWithEnv):
         identical."""
         assert isinstance(t1, Tree)
         assert isinstance(t2, Tree)
-        i = iter(self.__difftree.diff_trees(
-                ['-r', '-z'], t1.sha1, t2.sha1).split('\0'))
+        dt = self.__difftree.diff_trees(['-r', '-z'], t1.sha1, t2.sha1)
+        i = iter(dt.decode('utf-8').split('\0'))
         try:
             while True:
-                x = i.next()
+                x = next(i)
                 if not x:
                     continue
                 omode, nmode, osha1, nsha1, status = x[1:].split(' ')
-                fn1 = i.next()
+                fn1 = next(i)
                 if status[0] in ['C', 'R']:
-                    fn2 = i.next()
+                    fn2 = next(i)
                 else:
                     fn2 = fn1
                 yield (omode, nmode, self.get_blob(osha1),
@@ -824,8 +886,12 @@ class Index(RunWithEnv):
             self.delete()
         else:
             self.__filename = filename
-    env = property(lambda self: utils.add_dict(
-            self.__repository.env, { 'GIT_INDEX_FILE': self.__filename }))
+
+    @property
+    def env(self):
+        return utils.add_dict(self.__repository.env,
+                              {'GIT_INDEX_FILE': self.__filename})
+
     def read_tree(self, tree):
         self.run(['git', 'read-tree', tree.sha1]).no_output()
     def write_tree(self):
@@ -836,25 +902,26 @@ class Index(RunWithEnv):
             return self.__repository.get_tree(
                 self.run(['git', 'write-tree']).discard_stderr(
                     ).output_one_line())
-        except run.RunException:
+        except RunException:
             raise MergeException('Conflicting merge')
     def is_clean(self, tree):
         """Check whether the index is clean relative to the given treeish."""
         try:
             self.run(['git', 'diff-index', '--quiet', '--cached', tree.sha1]
                     ).discard_output()
-        except run.RunException:
+        except RunException:
             return False
         else:
             return True
-    def apply(self, patch_text, quiet):
+    def apply(self, patch_bytes, quiet):
         """In-index patch application, no worktree involved."""
         try:
-            r = self.run(['git', 'apply', '--cached']).raw_input(patch_text)
+            r = self.run(['git', 'apply', '--cached'])
+            r.encoding(None).raw_input(patch_bytes)
             if quiet:
                 r = r.discard_stderr()
             r.no_output()
-        except run.RunException:
+        except RunException:
             raise MergeException('Patch does not apply cleanly')
     def apply_treediff(self, tree1, tree2, quiet):
         """Apply the diff from C{tree1} to C{tree2} to the index."""
@@ -881,7 +948,7 @@ class Index(RunWithEnv):
         assert isinstance(base, Tree)
         assert isinstance(ours, Tree)
         assert isinstance(theirs, Tree)
-        assert current == None or isinstance(current, Tree)
+        assert current is None or isinstance(current, Tree)
 
         # Take care of the really trivial cases.
         if base == ours:
@@ -911,7 +978,7 @@ class Index(RunWithEnv):
         """The set of conflicting paths."""
         paths = set()
         for line in self.run(['git', 'ls-files', '-z', '--unmerged']
-                             ).raw_output().split('\0')[:-1]:
+                             ).output_lines('\0'):
             stat, path = line.split('\t', 1)
             paths.add(path)
         return paths
@@ -920,9 +987,18 @@ class Worktree(object):
     """Represents a git worktree (that is, a checked-out file tree)."""
     def __init__(self, directory):
         self.__directory = directory
-    env = property(lambda self: { 'GIT_WORK_TREE': '.' })
-    env_in_cwd = property(lambda self: { 'GIT_WORK_TREE': self.directory })
-    directory = property(lambda self: self.__directory)
+
+    @property
+    def env(self):
+        return {'GIT_WORK_TREE': '.'}
+
+    @property
+    def env_in_cwd(self):
+        return {'GIT_WORK_TREE': self.directory}
+
+    @property
+    def directory(self):
+        return self.__directory
 
 class CheckoutException(exception.StgException):
     """Exception raised when a checkout fails."""
@@ -935,11 +1011,23 @@ class IndexAndWorktree(RunWithEnvCwd):
     def __init__(self, index, worktree):
         self.__index = index
         self.__worktree = worktree
-    index = property(lambda self: self.__index)
-    env = property(lambda self: utils.add_dict(self.__index.env,
-                                               self.__worktree.env))
-    env_in_cwd = property(lambda self: self.__worktree.env_in_cwd)
-    cwd = property(lambda self: self.__worktree.directory)
+
+    @property
+    def index(self):
+        return self.__index
+
+    @property
+    def env(self):
+        return utils.add_dict(self.__index.env, self.__worktree.env)
+
+    @property
+    def env_in_cwd(self):
+        return self.__worktree.env_in_cwd
+
+    @property
+    def cwd(self):
+        return self.__worktree.directory
+
     def checkout_hard(self, tree):
         assert isinstance(tree, Tree)
         self.run(['git', 'read-tree', '--reset', '-u', tree.sha1]
@@ -954,7 +1042,7 @@ class IndexAndWorktree(RunWithEnvCwd):
                       '--exclude-per-directory=.gitignore',
                       old_tree.sha1, new_tree.sha1]
                      ).discard_output()
-        except run.RunException:
+        except RunException:
             raise CheckoutException('Index/workdir dirty')
     def merge(self, base, ours, theirs, interactive = False):
         assert isinstance(base, Tree)
@@ -975,7 +1063,7 @@ class IndexAndWorktree(RunWithEnvCwd):
                 else:
                     conflicts = [l for l in output if l.startswith('CONFLICT')]
                     raise MergeConflictException(conflicts)
-        except run.RunException, e:
+        except RunException:
             raise MergeException('Index/worktree dirty')
     def mergetool(self, files = ()):
         """Invoke 'git mergetool' on the current IndexAndWorktree to resolve
@@ -998,7 +1086,7 @@ class IndexAndWorktree(RunWithEnvCwd):
         assert isinstance(tree, Tree)
         return set(self.run_in_cwd(
                 ['git', 'diff-index', tree.sha1, '--name-only', '-z', '--']
-                + list(pathlimits)).raw_output().split('\0')[:-1])
+                + list(pathlimits)).output_lines('\0'))
     def update_index(self, paths):
         """Update the index with files from the worktree. C{paths} is an
         iterable of paths relative to the root of the repository."""
@@ -1008,8 +1096,8 @@ class IndexAndWorktree(RunWithEnvCwd):
     def worktree_clean(self):
         """Check whether the worktree is clean relative to index."""
         try:
-            self.run(['git', 'update-index', '--refresh']).discard_output()
-        except run.RunException:
+            self.run(['git', 'update-index', '--ignore-submodules', '--refresh']).discard_output()
+        except RunException:
             return False
         else:
             return True
@@ -1024,8 +1112,13 @@ class Branch(object):
         except KeyError:
             raise BranchException('%s: no such branch' % name)
 
-    name = property(lambda self: self.__name)
-    repository = property(lambda self: self.__repository)
+    @property
+    def name(self):
+        return self.__name
+
+    @property
+    def repository(self):
+        return self.__repository
 
     def __ref(self):
         return 'refs/heads/%s' % self.__name
@@ -1036,7 +1129,8 @@ class Branch(object):
         self.__repository.refs.set(self.__ref(), commit, msg)
 
     def set_parent_remote(self, name):
-        value = config.set('branch.%s.remote' % self.__name, name)
+        config.set('branch.%s.remote' % self.__name, name)
+
     def set_parent_branch(self, name):
         if config.get('branch.%s.remote' % self.__name):
             # Never set merge if remote is not set to avoid
@@ -1063,9 +1157,10 @@ class Branch(object):
 
 def diffstat(diff):
     """Return the diffstat of the supplied diff."""
-    return run.Run('git', 'apply', '--stat', '--summary'
-                   ).raw_input(diff).raw_output()
+    return (Run('git', 'apply', '--stat', '--summary')
+            .encoding(None).raw_input(diff)
+            .decoding('utf-8').raw_output())
 
 def clone(remote, local):
     """Clone a remote repository using 'git clone'."""
-    run.Run('git', 'clone', remote, local).run()
+    Run('git', 'clone', remote, local).run()
