@@ -1,4 +1,5 @@
-r"""This module contains functions and classes for manipulating
+# -*- coding: utf-8 -*-
+"""This module contains functions and classes for manipulating
 I{patch stack logs} (or just I{stack logs}).
 
 A stack log is a git branch. Each commit contains the complete state
@@ -99,12 +100,16 @@ The simplified log is exactly like the full log, except that its only
 parent is the (simplified) previous log entry, if any. It's purpose is
 mainly ease of visualization."""
 
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+from io import StringIO
 import re
-from stgit.lib import git, stack as libstack
+
 from stgit import utils
 from stgit.exception import StgException, StackException
+from stgit.lib import git, stack as libstack
 from stgit.out import out
-import StringIO
+
 
 class LogException(StgException):
     pass
@@ -112,19 +117,22 @@ class LogException(StgException):
 class LogParseException(LogException):
     pass
 
+
 def patch_file(repo, cd):
-    return repo.commit(git.BlobData(''.join(s + '\n' for s in [
-                    'Bottom: %s' % cd.parent.data.tree.sha1,
-                    'Top:    %s' % cd.tree.sha1,
-                    'Author: %s' % cd.author.name_email,
-                    'Date:   %s' % cd.author.date,
-                    '',
-                    cd.message,
-                    '',
-                    '---',
-                    '',
-                    repo.diff_tree(cd.parent.data.tree, cd.tree, ['-M']
-                                   ).strip()])))
+    metadata = '\n'.join([
+        'Bottom: %s' % cd.parent.data.tree.sha1,
+        'Top:    %s' % cd.tree.sha1,
+        'Author: %s' % cd.author.name_email,
+        'Date:   %s' % cd.author.date,
+        '',
+        cd.message,
+        '',
+        '---',
+        '',
+    ]).encode('utf-8')
+    diff = repo.diff_tree(cd.parent.data.tree, cd.tree, ['-M']).strip()
+    return repo.commit(git.BlobData(metadata + diff + b'\n'))
+
 
 def log_ref(branch):
     return 'refs/heads/%s.stgit' % branch
@@ -150,7 +158,7 @@ class LogEntry(object):
         return self.__simplified
     @property
     def prev(self):
-        if self.__prev != None and not isinstance(self.__prev, LogEntry):
+        if self.__prev is not None and not isinstance(self.__prev, LogEntry):
             self.__prev = self.from_commit(self.__repo, self.__prev)
         return self.__prev
     @property
@@ -165,8 +173,11 @@ class LogEntry(object):
             return self.patches[self.applied[-1]]
         else:
             return self.head
-    all_patches = property(lambda self: (self.applied + self.unapplied
-                                         + self.hidden))
+
+    @property
+    def all_patches(self):
+        return self.applied + self.unapplied + self.hidden
+
     @classmethod
     def from_stack(cls, prev, stack, message):
         return cls(
@@ -196,8 +207,10 @@ class LogEntry(object):
         if version > 1:
             raise LogException('Log is version %d, which is too new' % version)
         parsed = {}
+        key = None
         for line in metadata:
             if line.startswith(' '):
+                assert key is not None
                 parsed[key].append(line.strip())
             else:
                 key, val = [x.strip() for x in line.split(':', 1)]
@@ -213,7 +226,7 @@ class LogEntry(object):
         head = repo.get_commit(parsed['Head'])
         lists = { 'Applied': [], 'Unapplied': [], 'Hidden': [] }
         patches = {}
-        for lst in lists.keys():
+        for lst in lists:
             for entry in parsed[lst]:
                 pn, sha1 = [x.strip() for x in entry.split(':')]
                 lists[lst].append(pn)
@@ -229,14 +242,14 @@ class LogEntry(object):
         except KeyError:
             raise LogParseException('Not a stack log')
         (prev, head, applied, unapplied, hidden, patches
-         ) = cls.__parse_metadata(repo, meta.data.str)
+         ) = cls.__parse_metadata(repo, meta.data.bytes.decode('utf-8'))
         lg = cls(repo, prev, head, applied, unapplied, hidden, patches, message)
         lg.commit = commit
         return lg
     def __metadata_string(self):
-        e = StringIO.StringIO()
+        e = StringIO()
         e.write('Version: 1\n')
-        if self.prev == None:
+        if self.prev is None:
             e.write('Previous: None\n')
         else:
             e.write('Previous: %s\n' % self.prev.commit.sha1)
@@ -255,12 +268,12 @@ class LogEntry(object):
                                     for pn in self.unapplied + self.hidden)
         if self.applied:
             xp.add(self.patches[self.applied[-1]])
-        if self.prev != None:
+        if self.prev is not None:
             xp.add(self.prev.commit)
             xp -= set(self.prev.patches.values())
         return xp
     def __tree(self, metadata):
-        if self.prev == None:
+        if self.prev is None:
             def pf(c):
                 return patch_file(self.__repo, c.data)
         else:
@@ -269,15 +282,15 @@ class LogEntry(object):
             # Map from Commit object to patch_file() results taken
             # from the previous log entry.
             c2b = dict((self.prev.patches[pn], pf) for pn, pf
-                       in prev_patch_tree.data.entries.iteritems())
+                       in prev_patch_tree.data.entries.items())
             def pf(c):
                 r = c2b.get(c, None)
                 if not r:
                     r = patch_file(self.__repo, c.data)
                 return r
-        patches = dict((pn, pf(c)) for pn, c in self.patches.iteritems())
+        patches = dict((pn, pf(c)) for pn, c in self.patches.items())
         return self.__repo.commit(git.TreeData({
-                    'meta': self.__repo.commit(git.BlobData(metadata)),
+                    'meta': self.__repo.commit(git.BlobData(metadata.encode('utf-8'))),
                     'patches': self.__repo.commit(git.TreeData(patches)) }))
     def write_commit(self):
         metadata = self.__metadata_string()
@@ -285,7 +298,7 @@ class LogEntry(object):
         self.__simplified = self.__repo.commit(git.CommitData(
                 tree = tree, message = self.message,
                 parents = [prev.simplified for prev in [self.prev]
-                           if prev != None]))
+                           if prev is not None]))
         parents = list(self.__parents())
         while len(parents) >= self.__max_parents:
             g = self.__repo.commit(git.CommitData(
@@ -299,7 +312,7 @@ class LogEntry(object):
 def get_log_entry(repo, ref, commit):
     try:
         return LogEntry.from_commit(repo, commit)
-    except LogException, e:
+    except LogException as e:
         raise LogException('While reading log from %s: %s' % (ref, e))
 
 def same_state(log1, log2):
@@ -321,7 +334,7 @@ def log_entry(stack, msg):
         else:
             last_log = None
         new_log = LogEntry.from_stack(last_log, stack, msg)
-    except LogException, e:
+    except LogException as e:
         out.warn(str(e), 'No log entry written.')
         return
     if last_log and same_state(last_log, new_log):
@@ -420,7 +433,7 @@ def reset_stack_partially(trans, iw, state, only_patches):
     # In one go, do all the popping we have to in order to pop the
     # patches we're going to delete or modify.
     def mod(pn):
-        if not pn in only_patches:
+        if pn not in only_patches:
             return False
         if pn in to_delete:
             return True
