@@ -1,8 +1,12 @@
-import os.path
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import os
+
 from stgit import utils
-from stgit.out import out
 from stgit.config import config
 from stgit.exception import StackException
+from stgit.out import out
 
 # The current StGit metadata format version.
 FORMAT_VERSION = 2
@@ -64,8 +68,11 @@ def update_to_current_format_version(repository, branch):
         patch_dir = os.path.join(branch_dir, 'patches')
         mkdir(patch_dir)
         refs_base = 'refs/patches/%s' % branch
-        for patch in (file(os.path.join(branch_dir, 'unapplied')).readlines()
-                      + file(os.path.join(branch_dir, 'applied')).readlines()):
+        with open(os.path.join(branch_dir, 'unapplied')) as f:
+            patches = f.readlines()
+        with open(os.path.join(branch_dir, 'applied')) as f:
+            patches.extend(f.readlines())
+        for patch in patches:
             patch = patch.strip()
             os.rename(os.path.join(branch_dir, patch),
                       os.path.join(patch_dir, patch))
@@ -100,7 +107,7 @@ def update_to_current_format_version(repository, branch):
 
     # Make sure we're at the latest version.
     fv = get_format_version()
-    if not fv in [None, FORMAT_VERSION]:
+    if fv not in [None, FORMAT_VERSION]:
         raise StackException('Branch %s is at format version %d, expected %d'
                              % (branch, fv, FORMAT_VERSION))
-    return fv != None # true if branch is initialized
+    return fv is not None # true if branch is initialized
