@@ -1,14 +1,17 @@
+# -*- coding: utf-8 -*-
 """The L{StackTransaction} class makes it possible to make complex
 updates to an StGit stack in a safe and convenient way."""
 
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+from itertools import takewhile
 import atexit
-import itertools as it
 
 from stgit import exception, utils
-from stgit.utils import any
-from stgit.out import *
-from stgit.lib import git, log
 from stgit.config import config
+from stgit.lib import git, log
+from stgit.out import out
+
 
 class TransactionException(exception.StgException):
     """Exception raised when something goes wrong with a
@@ -105,44 +108,78 @@ class StackTransaction(object):
             self.__assert_head_top_equal()
         if check_clean_iw:
             self.__assert_index_worktree_clean(check_clean_iw)
-    stack = property(lambda self: self.__stack)
-    patches = property(lambda self: self.__patches)
-    def __set_applied(self, val):
-        self.__applied = list(val)
-    applied = property(lambda self: self.__applied, __set_applied)
-    def __set_unapplied(self, val):
-        self.__unapplied = list(val)
-    unapplied = property(lambda self: self.__unapplied, __set_unapplied)
-    def __set_hidden(self, val):
-        self.__hidden = list(val)
-    hidden = property(lambda self: self.__hidden, __set_hidden)
-    all_patches = property(lambda self: (self.__applied + self.__unapplied
-                                         + self.__hidden))
-    def __set_base(self, val):
+
+    @property
+    def stack(self):
+        return self.__stack
+
+    @property
+    def patches(self):
+        return self.__patches
+
+    @property
+    def applied(self):
+        return self.__applied
+
+    @applied.setter
+    def applied(self, value):
+        self.__applied = list(value)
+
+    @property
+    def unapplied(self):
+        return self.__unapplied
+
+    @unapplied.setter
+    def unapplied(self, value):
+        self.__unapplied = list(value)
+
+    @property
+    def hidden(self):
+        return self.__hidden
+
+    @hidden.setter
+    def hidden(self, value):
+        self.__hidden = list(value)
+
+    @property
+    def all_patches(self):
+        return self.__applied + self.__unapplied + self.__hidden
+
+    @property
+    def base(self):
+        return self.__base
+
+    @base.setter
+    def base(self, value):
         assert (not self.__applied
-                or self.patches[self.applied[0]].data.parent == val)
-        self.__base = val
-    base = property(lambda self: self.__base, __set_base)
+                or self.patches[self.applied[0]].data.parent == value)
+        self.__base = value
+
     @property
     def temp_index(self):
         if not self.__temp_index:
             self.__temp_index = self.__stack.repository.temp_index()
             atexit.register(self.__temp_index.delete)
         return self.__temp_index
+
     @property
     def top(self):
         if self.__applied:
             return self.__patches[self.__applied[-1]]
         else:
             return self.__base
-    def __get_head(self):
+
+    @property
+    def head(self):
         if self.__bad_head:
             return self.__bad_head
         else:
             return self.top
-    def __set_head(self, val):
-        self.__bad_head = val
-    head = property(__get_head, __set_head)
+
+    @head.setter
+    def head(self, value):
+        self.__bad_head = value
+
     def __assert_head_top_equal(self):
         if not self.__stack.head_top_equal():
             out.error(
@@ -163,12 +200,12 @@ class StackTransaction(object):
             # there are no unresolved conflicts. Conflicts
             # conceptually "belong" to the topmost patch, and just
             # carrying them along to another patch is confusing.
-            if (self.__allow_conflicts(self) or iw == None
+            if (self.__allow_conflicts(self) or iw is None
                 or not iw.index.conflicts()):
                 return
             out.error('Need to resolve conflicts first')
             self.__abort()
-        assert iw != None
+        assert iw is not None
         if self.__discard_changes:
             iw.checkout_hard(tree)
         else:
@@ -180,8 +217,8 @@ class StackTransaction(object):
             'Command aborted (all changes rolled back)')
     def __check_consistency(self):
         remaining = set(self.all_patches)
-        for pn, commit in self.__patches.iteritems():
-            if commit == None:
+        for pn, commit in self.__patches.items():
+            if commit is None:
                 assert self.__stack.patches.exists(pn)
             else:
                 assert pn in remaining
@@ -217,10 +254,10 @@ class StackTransaction(object):
 
         # Write patches.
         def write(msg):
-            for pn, commit in self.__patches.iteritems():
+            for pn, commit in self.__patches.items():
                 if self.__stack.patches.exists(pn):
                     p = self.__stack.patches.get(pn)
-                    if commit == None:
+                    if commit is None:
                         p.delete()
                     else:
                         p.set_commit(commit, msg)
@@ -261,7 +298,7 @@ class StackTransaction(object):
         other patches that had to be popped to accomplish this. Always
         succeeds."""
         popped = []
-        for i in xrange(len(self.applied)):
+        for i in range(len(self.applied)):
             if p(self.applied[i]):
                 popped = self.applied[i:]
                 del self.applied[i:]
@@ -278,7 +315,7 @@ class StackTransaction(object):
         succeeds."""
         popped = []
         all_patches = self.applied + self.unapplied + self.hidden
-        for i in xrange(len(self.applied)):
+        for i in range(len(self.applied)):
             if p(self.applied[i]):
                 popped = self.applied[i:]
                 del self.applied[i:]
@@ -317,7 +354,7 @@ class StackTransaction(object):
         s = ''
         merge_conflict = False
         if not tree:
-            if iw == None:
+            if iw is None:
                 self.__halt('%s does not apply cleanly' % pn)
             try:
                 self.__checkout(ours, iw, allow_bad_head = False)
@@ -325,17 +362,17 @@ class StackTransaction(object):
                 self.__halt('Index/worktree dirty')
             try:
                 interactive = (allow_interactive and
-                               config.get('stgit.autoimerge') == 'yes')
+                               config.getbool('stgit.autoimerge'))
                 iw.merge(base, ours, theirs, interactive = interactive)
                 tree = iw.index.write_tree()
                 self.__current_tree = tree
                 s = 'modified'
-            except git.MergeConflictException, e:
+            except git.MergeConflictException as e:
                 tree = ours
                 merge_conflict = True
                 self.__conflicts = e.conflicts
                 s = 'conflict'
-            except git.MergeException, e:
+            except git.MergeException as e:
                 self.__halt(str(e))
         cd = cd.set_tree(tree)
         if any(getattr(cd, a) != getattr(orig_cd, a) for a in
@@ -402,8 +439,8 @@ class StackTransaction(object):
         """Push and pop patches to attain the given ordering."""
         if hidden is None:
             hidden = self.hidden
-        common = len(list(it.takewhile(lambda (a, b): a == b,
-                                       zip(self.applied, applied))))
+        common = len(list(takewhile(lambda a: a[0] == a[1],
+                                    zip(self.applied, applied))))
         to_pop = set(self.applied[common:])
         self.pop_patches(lambda pn: pn in to_pop)
         for pn in applied[common:]:
