@@ -1,5 +1,18 @@
-"""Basic quilt-like functionality
-"""
+# -*- coding: utf-8 -*-
+"""Basic quilt-like functionality"""
+
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import os
+import sys
+import traceback
+
+from stgit import argparse, run, utils
+from stgit.compat import environ_get, fsdecode_utf8
+from stgit.config import config
+from stgit.out import out
+from stgit.pager import pager
+import stgit.commands
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -17,12 +30,6 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-import sys, os, traceback
-
-import stgit.commands
-from stgit.out import *
-from stgit import argparse, run, utils
-from stgit.config import config
 
 class CommandAlias(object):
     def __init__(self, name, command):
@@ -57,7 +64,7 @@ class Commands(dict):
         """Return the canonical name for a possibly-shortenned
         command name.
         """
-        candidates = [cmd for cmd in self.keys() if cmd.startswith(key)]
+        candidates = [cmd for cmd in self if cmd.startswith(key)]
 
         if not candidates:
             out.error('Unknown command: %s' % key,
@@ -69,7 +76,7 @@ class Commands(dict):
             sys.exit(utils.STGIT_GENERAL_ERROR)
 
         return candidates[0]
-        
+
     def __getitem__(self, key):
         cmd_mod = self.get(key) or self.get(self.canonical_cmd(key))
         if is_cmd_alias(cmd_mod):
@@ -80,16 +87,16 @@ class Commands(dict):
 cmd_list = stgit.commands.get_commands()
 append_alias_commands(cmd_list)
 commands = Commands((cmd, mod) for cmd, (mod, kind, help)
-                    in cmd_list.iteritems())
+                    in cmd_list.items())
 
 def print_help():
-    print 'usage: %s <command> [options]' % os.path.basename(sys.argv[0])
-    print
-    print 'Generic commands:'
-    print '  help        print the detailed command usage'
-    print '  version     display version information'
-    print '  copyright   display copyright information'
-    print
+    print('usage: %s <command> [options]' % os.path.basename(sys.argv[0]))
+    print()
+    print('Generic commands:')
+    print('  help        print the detailed command usage')
+    print('  version     display version information')
+    print('  copyright   display copyright information')
+    print()
     stgit.commands.pretty_command_list(cmd_list, sys.stdout)
 
 #
@@ -100,12 +107,13 @@ def _main():
     """
     global prog
 
+    sys.argv = list(map(fsdecode_utf8, sys.argv))
+
     prog = os.path.basename(sys.argv[0])
 
     if len(sys.argv) < 2:
-        print >> sys.stderr, 'usage: %s <command>' % prog
-        print >> sys.stderr, \
-              '  Try "%s --help" for a list of supported commands' % prog
+        print('usage: %s <command>' % prog, file=sys.stderr)
+        print('  Try "%s --help" for a list of supported commands' % prog, file=sys.stderr)
         sys.exit(utils.STGIT_GENERAL_ERROR)
 
     cmd = sys.argv[1]
@@ -120,34 +128,29 @@ def _main():
     if cmd == 'help':
         if len(sys.argv) == 3 and not sys.argv[2] in ['-h', '--help']:
             cmd = commands.canonical_cmd(sys.argv[2])
-            if not cmd in commands:
-                out.error('%s help: "%s" command unknown' % (prog, cmd))
-                sys.exit(utils.STGIT_GENERAL_ERROR)
-
             sys.argv[0] += ' %s' % cmd
             command = commands[cmd]
             parser = argparse.make_option_parser(command)
             if is_cmd_alias(command):
                 parser.remove_option('-h')
-            from pydoc import pager
-            pager(parser.format_help())
+            pager(parser.format_help().encode())
         else:
             print_help()
         sys.exit(utils.STGIT_SUCCESS)
     if cmd in ['-v', '--version', 'version']:
-        from stgit.version import version
-        print 'Stacked GIT %s' % version
+        from stgit.version import get_version
+        print('Stacked GIT %s' % get_version())
         os.system('git --version')
-        print 'Python version %s' % sys.version
+        print('Python version %s' % sys.version)
         sys.exit(utils.STGIT_SUCCESS)
     if cmd in ['copyright']:
-        print __copyright__
+        print(__copyright__)
         sys.exit(utils.STGIT_SUCCESS)
 
     # re-build the command line arguments
     cmd = commands.canonical_cmd(cmd)
     sys.argv[0] += ' %s' % cmd
-    del(sys.argv[1])
+    del sys.argv[1]
 
     command = commands[cmd]
     if is_cmd_alias(command):
@@ -158,13 +161,16 @@ def _main():
 
     # These modules are only used from this point onwards and do not
     # need to be imported earlier
+    try:
+        from configparser import ParsingError, NoSectionError
+    except ImportError:
+        from ConfigParser import ParsingError, NoSectionError
     from stgit.exception import StgException
     from stgit.config import config_setup
-    from ConfigParser import ParsingError, NoSectionError
     from stgit.stack import Series
 
     try:
-        debug_level = int(os.environ.get('STGIT_DEBUG_LEVEL', 0))
+        debug_level = int(environ_get('STGIT_DEBUG_LEVEL', 0))
     except ValueError:
         out.error('Invalid STGIT_DEBUG_LEVEL environment variable')
         sys.exit(utils.STGIT_GENERAL_ERROR)
@@ -182,7 +188,7 @@ def _main():
                 command.crt_series = Series()
 
         ret = command.func(parser, options, args)
-    except (StgException, IOError, ParsingError, NoSectionError), err:
+    except (StgException, IOError, ParsingError, NoSectionError) as err:
         directory.write_log(cmd)
         if debug_level > 0:
             traceback.print_exc(file=sys.stderr)
