@@ -1,10 +1,19 @@
-"""Common utility functions
-"""
+# -*- coding: utf-8 -*-
+"""Common utility functions"""
 
-import errno, os, os.path, re, sys, tempfile
-from stgit.exception import *
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+from io import open
+import errno
+import os
+import re
+import sys
+import tempfile
+
+from stgit.compat import text, environ_get
 from stgit.config import config
-from stgit.out import *
+from stgit.exception import StgException
+from stgit.out import out
 
 __copyright__ = """
 Copyright (C) 2005, Catalin Marinas <catalin.marinas@gmail.com>
@@ -22,72 +31,64 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/.
 """
 
-def mkdir_file(filename, mode):
+
+def mkdir_file(filename, mode, encoding='utf-8'):
     """Opens filename with the given mode, creating the directory it's
     in if it doesn't already exist."""
     create_dirs(os.path.dirname(filename))
-    return file(filename, mode)
+    return open(filename, mode, encoding=encoding)
 
-def read_strings(filename):
+def read_strings(filename, encoding='utf-8'):
     """Reads the lines from a file
     """
-    f = file(filename, 'r')
-    lines = [line.strip() for line in f.readlines()]
-    f.close()
-    return lines
+    with open(filename, encoding=encoding) as f:
+        return [line.strip() for line in f.readlines()]
 
-def read_string(filename, multiline = False):
+def read_string(filename, multiline=False, encoding='utf-8'):
     """Reads the first line from a file
     """
-    f = file(filename, 'r')
-    if multiline:
-        result = f.read()
-    else:
-        result = f.readline().strip()
-    f.close()
-    return result
+    with open(filename, encoding=encoding) as f:
+        if multiline:
+            return f.read()
+        else:
+            return f.readline().strip()
 
-def write_strings(filename, lines):
+def write_strings(filename, lines, encoding='utf-8'):
     """Write 'lines' sequence to file
     """
-    f = file(filename, 'w+')
-    f.writelines([line + '\n' for line in lines])
-    f.close()
+    with open(filename, 'w+', encoding=encoding) as f:
+        for line in lines:
+            print(line, file=f)
 
-def write_string(filename, line, multiline = False):
+def write_string(filename, line, multiline=False, encoding='utf-8'):
     """Writes 'line' to file and truncates it
     """
-    f = mkdir_file(filename, 'w+')
-    if multiline:
-        f.write(line)
-    else:
-        print >> f, line
-    f.close()
+    with mkdir_file(filename, 'w+', encoding) as f:
+        line = text(line)
+        print(line, end='' if multiline else '\n', file=f)
 
-def append_strings(filename, lines):
+def append_strings(filename, lines, encoding='utf-8'):
     """Appends 'lines' sequence to file
     """
-    f = mkdir_file(filename, 'a+')
-    for line in lines:
-        print >> f, line
-    f.close()
+    with mkdir_file(filename, 'a+', encoding) as f:
+        for line in lines:
+            print(line, file=f)
 
-def append_string(filename, line):
+def append_string(filename, line, encoding='utf-8'):
     """Appends 'line' to file
     """
-    f = mkdir_file(filename, 'a+')
-    print >> f, line
-    f.close()
+    with mkdir_file(filename, 'a+', encoding) as f:
+        print(line, file=f)
 
-def insert_string(filename, line):
+def insert_string(filename, line, encoding='utf-8'):
     """Inserts 'line' at the beginning of the file
     """
-    f = mkdir_file(filename, 'r+')
-    lines = f.readlines()
-    f.seek(0); f.truncate()
-    print >> f, line
-    f.writelines(lines)
-    f.close()
+    with mkdir_file(filename, 'r+', encoding) as f:
+        lines = f.readlines()
+        f.seek(0)
+        f.truncate()
+        print(line, file=f)
+        f.writelines(lines)
 
 def create_empty_file(name):
     """Creates an empty file
@@ -149,7 +150,7 @@ def create_dirs(directory):
         create_dirs(os.path.dirname(directory))
         try:
             os.mkdir(directory)
-        except OSError, e:
+        except OSError as e:
             if e.errno != errno.EEXIST:
                 raise e
 
@@ -170,11 +171,11 @@ class EditorException(StgException):
     pass
 
 def get_editor():
-    for editor in [os.environ.get('GIT_EDITOR'),
+    for editor in [environ_get('GIT_EDITOR'),
                    config.get('stgit.editor'), # legacy
                    config.get('core.editor'),
-                   os.environ.get('VISUAL'),
-                   os.environ.get('EDITOR'),
+                   environ_get('VISUAL'),
+                   environ_get('EDITOR'),
                    'vi']:
         if editor:
             return editor
@@ -185,7 +186,7 @@ def call_editor(filename):
     out.start('Invoking the editor: "%s"' % cmd)
     err = os.system(cmd)
     if err:
-        raise EditorException, 'editor failed, exit code: %d' % err
+        raise EditorException('editor failed, exit code: %d' % err)
     out.done()
 
 def get_hook(repository, hook_name, extra_env={}):
@@ -211,21 +212,25 @@ def get_hook(repository, hook_name, extra_env={}):
 
         default_iw.run(argv, extra_env).run()
 
-    hook.__name__ = hook_name
+    hook.__name__ = str(hook_name)
     return hook
 
 def run_hook_on_string(hook, s, *args):
     if hook is not None:
-        temp = tempfile.NamedTemporaryFile('wb', delete=False)
+        temp = tempfile.NamedTemporaryFile('w', delete=False)
         try:
             try:
                 temp.write(s)
             finally:
                 temp.close()
 
-            hook(temp.name, *args)
+            if sys.version_info[0] >= 3:
+                hook_path = temp.name
+            else:
+                hook_path = temp.name.decode(sys.getfilesystemencoding())
+            hook(hook_path, *args)
 
-            output = open(temp.name, 'rb')
+            output = open(hook_path, 'r')
             try:
                 s = output.read()
             finally:
@@ -235,14 +240,21 @@ def run_hook_on_string(hook, s, *args):
 
     return s
 
-def edit_string(s, filename):
-    f = file(filename, 'w')
-    f.write(s)
-    f.close()
+def edit_string(s, filename, encoding='utf-8'):
+    with open(filename, 'w', encoding=encoding) as f:
+        f.write(s)
     call_editor(filename)
-    f = file(filename)
-    s = f.read()
-    f.close()
+    with open(filename, encoding=encoding) as f:
+        s = f.read()
+    os.remove(filename)
+    return s
+
+def edit_bytes(s, filename):
+    with open(filename, 'wb') as f:
+        f.write(s)
+    call_editor(filename)
+    with open(filename, 'rb') as f:
+        s = f.read()
     os.remove(filename)
     return s
 
@@ -276,7 +288,7 @@ def patch_name_from_msg(msg):
         name_len = 30
 
     subject_line = msg.split('\n', 1)[0].lstrip().lower()
-    words = re.sub('[\W]+', ' ', subject_line).split()
+    words = re.sub(r'[\W]+', ' ', subject_line).split()
 
     # use loop to avoid truncating the last name
     name = words and words[0] or 'unknown'
@@ -297,25 +309,6 @@ def make_patch_name(msg, unacceptable, default_name = 'patch'):
         patchname = default_name
     return find_patch_name(patchname, unacceptable)
 
-# any and all functions are builtin in Python 2.5 and higher, but not
-# in 2.4.
-if 'any' in dir(__builtins__):
-    any = getattr(__builtins__, 'any')
-else:
-    def any(bools):
-        for b in bools:
-            if b:
-                return True
-        return False
-
-if 'all' in dir(__builtins__):
-    all = getattr(__builtins__, 'all')
-else:
-    def all(bools):
-        for b in bools:
-            if not b:
-                return False
-        return True
 
 def add_sign_line(desc, sign_str, name, email):
     if not sign_str:
@@ -346,7 +339,7 @@ def parse_name_email_date(address):
     """Return a tuple consisting of the name, email and date parsed
     from a 'name <email> date' string."""
     address = re.sub(r'[\\"]', r'\\\g<0>', address)
-    str_list = re.findall('^(.*)\s*<(.*)>\s*(.*)\s*$', address)
+    str_list = re.findall(r'^(.*)\s*<(.*)>\s*(.*)\s*$', address)
     if not str_list:
         return None
     return str_list[0]
