@@ -16,6 +16,29 @@ test_expect_success \
     stg refresh
     '
 
+test_expect_success \
+    'Invalid arguments' \
+    '
+    command_error stg delete --top foo 2>&1 |
+    grep -e "Either --top or patches must be specified"
+    '
+
+test_expect_success \
+    'Attempt delete --top with none applied' \
+    '
+    stg pop &&
+    command_error stg delete --top 2>&1 |
+    grep -e "No patches applied" &&
+    stg push
+    '
+
+test_expect_success \
+    'No patches specified' \
+    '
+    command_error stg delete 2>&1 |
+    grep -e "No patches specified"
+    '
+
 test_expect_success \
     'Try to delete a non-existing patch' \
     '
