@@ -97,6 +97,29 @@ test_expect_success \
     stg delete ..
     '
 
+test_expect_success \
+    'Apply a patch from latin1-encoded email specifying utf-8 charset' \
+    '
+    iconv -f UTF8 -t LATIN1 -o email-latin1 $STG_ROOT/t/t1800-import/email-8bit &&
+    stg import -m email-latin1 &&
+    [ $(git cat-file -p $(stg id) \
+        | grep -c "tree cf0f9884fdb30bca14d2411e1711f6ae413c9213") = 1 ] &&
+    [ $(git cat-file -p $(stg id) \
+        | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
+    stg delete ..
+    '
+
+test_expect_success \
+    'Apply a patch from email with quoted "From" header' \
+    '
+    stg import -m $STG_ROOT/t/t1800-import/email-quoted-from &&
+    [ $(git cat-file -p $(stg id) \
+        | grep -c "tree 030be42660323ff2a1958f9ee79589a4f3fbee2f") = 1 ] &&
+    [ $(git cat-file -p $(stg id) \
+        | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
+    stg delete ..
+    '
+
 test_expect_success \
     'Apply a patch from a QP-encoded e-mail' \
     '
@@ -155,7 +178,7 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "tree e96b1fba2160890ff600b675d7140d46b022b155") = 1 ] &&
     rm $STG_ROOT/t/t1800-import/bzip2-git-diff &&
-    stg delete .. 
+    stg delete ..
     '
 test_expect_success \
     'Apply a bzip2 patch with a .bz2 suffix' \
@@ -166,7 +189,7 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "tree e96b1fba2160890ff600b675d7140d46b022b155") = 1 ] &&
     rm $STG_ROOT/t/t1800-import/git-diff.bz2 &&
-    stg delete .. 
+    stg delete ..
     '
 
 test_expect_success \
@@ -204,4 +227,133 @@ test_expect_success \
     rm $STG_ROOT/t/t1800-import/jabberwocky.tar.bz2
     '
 
+test_expect_success \
+    'Import from git format-patch output' '
+    (
+        test_create_repo upstream &&
+        cd upstream &&
+        echo "something" > some.txt &&
+        git add some.txt &&
+        git commit -m "Add something"
+        stg init
+    ) &&
+    (
+        git clone upstream downstream
+        cd downstream &&
+        echo "else µ" >> some.txt &&
+        git commit -a --author "Éd <ed@example.com>" -m "something else" &&
+        git format-patch --stdout HEAD~1 > ../downstream.mbox
+    ) &&
+    (
+        cd upstream &&
+        stg import --mbox ../downstream.mbox &&
+        stg top | grep "something-else" &&
+        grep "else µ" some.txt
+    )
+    '
+
+test_expect_success \
+    'Import with author options' \
+    '
+    stg show | grep -e "Author: Clark Williams <williams@redhat.com>" &&
+    stg delete --top --spill &&
+    stg diff > some.patch &&
+    git reset jabberwocky.txt &&
+    git checkout jabberwocky.txt &&
+    stg import --authname "Some Author" \
+               --authemail "some@example.com" \
+               --authdate 2005-04-07T22:13:13 \
+               --stripname \
+               some.patch &&
+    stg show | grep -e "Author: Some Author <some@example.com>" &&
+    stg show | grep -E "Date: +Thu Apr 7 22:13:13 2005 \+0000" &&
+    stg delete --top
+    '
+
+test_expect_success \
+    'Import with bad author_date option' \
+    '
+    stg delete --top &&
+    command_error stg import --authdate "a long time ago" some.patch 2>&1 |
+    grep -e "\"a long time ago\" is not a valid date"
+    '
+
+test_expect_success \
+    'Import from stdin' \
+    '
+    cat some.patch |
+    stg import --name xxx \
+               --authname "Some Author" \
+               --authemail "some@example.com" &&
+    test "$(echo $(stg top))" = "xxx" &&
+    stg show | grep -e "Author: Some Author <some@example.com>"
+    '
+
+test_expect_success \
+    'Replace existing patch' \
+    '
+    stg pop xxx &&
+    stg import --replace \
+               --name xxx \
+               --author "Different Author <diff@example.com>" \
+               some.patch &&
+    test "$(echo $(stg top))" = "xxx" &&
+    stg show | grep -e "Author: Different Author <diff@example.com>"
+    '
+
+test_expect_success \
+    'Ignore patch reapplication' \
+    '
+    stg top | grep -e "xxx" &&
+    stg import --ignore --name xxx some.patch &&
+    test "$(echo $(stg top))" = "xxx" &&
+    stg show | grep -e "Author: Different Author <diff@example.com>" &&
+    stg delete --top
+    '
+
+test_expect_success \
+    'Import from stdin no name' \
+    '
+    cat some.patch |
+    stg import --ignore --author "Some Author <some@example.com>" &&
+    stg top | grep -e "unknown" &&
+    stg show | grep -e "Author: Some Author <some@example.com>" &&
+    stg delete --top
+    '
+
+test_expect_success \
+    'Import empty patch with sign-off' \
+    '
+    echo "" |
+    stg import -n empty --sign 2>&1 |
+    grep -e "No diff found, creating empty patch" &&
+    stg show | grep -e "Signed-off-by: C Ó Mitter <committer@example.com>" &&
+    stg top | grep -e "empty" &&
+    stg clean &&
+    stg top | grep -v -e "empty"
+    '
+
+test_expect_success \
+    'Import series from stdin' \
+    '
+    echo "some.patch" |
+    stg import --series &&
+    stg top | grep -e "some.patch" &&
+    stg delete --top
+    '
+
+test_expect_success \
+    'Attempt url' \
+    '
+    command_error stg import --url 2>&1 |
+    grep -e "URL argument required"
+    '
+
+test_expect_success \
+    'Too many arguments' \
+    '
+    command_error stg import some.patch some.patch 2>&1 |
+    grep -e "incorrect number of arguments"
+    '
+
 test_done
